# -*- coding: utf-8 -*-
# ---------------------------------------------------------
# Name:         Conf
# Purpose:      Show configuration or set configuration
# Author:
# Created:      2022-07-18
# ---------------------------------------------------------

from reptor.lib.plugins.ConfBase import ConfBase

from reptor.utils.table import make_table


class Conf(ConfBase):
    """
    Offers you a way to write a config file for reptor,
    allows you to quickly see the configuration without touching
    the config file
    """

    meta = {"name": "Config", "summary": "Shows config and sets config"}

    def __init__(self, **kwargs):
        super().__init__(**kwargs)
        self.arg_show = kwargs.get("show")

    @classmethod
    def add_arguments(cls, parser, plugin_filepath=None):
        super().add_arguments(parser, plugin_filepath)
        config_parser = parser.add_mutually_exclusive_group()
        config_parser.add_argument(
            "--show", help="Shows current connection settings", action="store_true"
        )

    def _show_config(self):
        self.display("Configuration Overview")

        table = make_table(["Setting", "Value"])

        table.add_row("Server", self.reptor.get_config().get_server())

        project_id = self.reptor.get_config().get_project_id()
        table.add_row("Project ID", project_id or "Writing globally.")

        self.console.print(table)

    def run(self):
        if self.arg_show:
            self._show_config()
        else:
            self.reptor.get_config().get_config_from_user()


loader = Conf



import json

from reptor.models.User import User


class TestUserModelParsing:
    example_user = """
{
            "id": "e79abf41-5b7c-4c1f-826e-040f34eaf6b4",
            "created": "2023-05-19T08:19:23.679429Z",
            "updated": "2023-05-19T08:19:23.680725Z",
            "last_login": null,
            "is_active": true,
            "username": "rs",
            "name": "Richard Schwabe",
            "title_before": null,
            "first_name": "Richard",
            "middle_name": null,
            "last_name": "Schwabe",
            "title_after": null,
            "email": null,
            "phone": null,
            "mobile": null,
            "scope": [
                "admin",
                "template_editor",
                "designer",
                "user_manager"
            ],
            "is_superuser": true,
            "is_designer": false,
            "is_template_editor": false,
            "is_user_manager": false,
            "is_guest": false,
            "is_system_user": false,
            "is_global_archiver": false,
            "is_mfa_enabled": false,
            "can_login_local": true,
            "can_login_sso": false
        }
        """

    def test_user_parsing(self):
        api_test_data = json.loads(self.example_user)

        test_user = User(api_test_data)

        assert test_user.id == "e79abf41-5b7c-4c1f-826e-040f34eaf6b4"
        assert test_user.name == "Richard Schwabe"
        assert test_user.first_name == "Richard"
        assert test_user.last_name == "Schwabe"
        assert test_user.is_superuser == True
        assert test_user.scope



import json


class ModelBase:
    def __str__(self):
        return self.__repr__()

    def __repr__(self):
        return json.dumps(
            self.__dict__,
            indent=2,
            default=str,
        )

    def toJson(self):
        return json.dumps(self, default=lambda o: o.__dict__)



from rich import box
from rich.table import Table

from typing import List, Optional


def make_table(columns: List[str], caption:  Optional[str] = None) -> Table:
    """This is a simpel helper function to ensure that all tables look the same
    Simply provide the columns you want to print out, as well as a specific caption.

    You can then use `table.add_row(finding.title, template.id)` to add the data
    according to your columns.

    In case you want to highlight a specific cell make use of the BBCode like
    syntax, such as [red]Important[/red]

    At the end call `reptor_console.print(table)` with your final table


    Args:
        columns (list[str]): Columns
        caption (str | None, optional): a Table caption at the bottom of the table. Defaults to None.

    Returns:
        Table: The raw table, without data, but with all styles set
    """

    table = Table(show_header=True, header_style="bold blue")
    if caption:
        table.caption = caption
    table.row_styles = ["none", "dim"]
    table.border_style = "bright_blue"
    table.box = box.SQUARE
    table.pad_edge = False

    for column in columns:
        table.add_column(column)

    return table



from reptor.lib.plugins.ToolBase import ToolBase


class MYMODULENAME(ToolBase):
    """
    Example Commands:
        TODO: Add example commands
    """

    meta = {
        "author": "",
        "name": "",
        "version": "0.1",
        "license": "MIT",
        "tags": [],
        "summary": "",
    }

    def __init__(self, **kwargs):
        super().__init__(**kwargs)
        self.notename = kwargs.get("notename", "MYMODULENAME")
        self.note_icon = "📃"
        self.arg_foo = kwargs.get("foo")

    @classmethod
    def add_arguments(cls, parser, plugin_filepath=None):
        """Any arguments added in here are accessible via kwargs in the constructor"""
        super().add_arguments(parser, plugin_filepath=plugin_filepath)

        # Simple Toggle
        parser.add_argument(
            "--foo", help="Create as numeric list", action="store_true", default=False
        )

        # Simple Integer only valid from a selection of integers
        parser.add_argument(
            "--bar",
            help="More complex example",
            action="store",
            type=int,
            choices=[1, 2, 3, 4, 5, 6],
            default=None,
        )

    def parse_xml(self, xml_root):
        """This is called automatically if the user provices --format xml
        For more infos look at the parent parse() method in ToolBase
        """
        super().parse_xml()

    def parse_json(self):
        """This is called automatically if the user provices --format json
        For more infos look at the parent parse() method in ToolBase
        """
        super().parse_json()

    def parse_csv(self):
        """This is called automatically if the user provices --format csv
        For more infos look at the parent parse() method in ToolBase
        """
        ...

    def preprocess_for_template(self):
        return super().preprocess_for_template()


loader = MYMODULENAME



def guess_filetype(content):
    ext = None
    if b"PNG" in content[:4]:
        ext = "png"
    elif b"JFIF" in content[:20]:
        ext = "jpg"
    elif b"GIF" in content[:3]:
        ext = "gif"
    elif b"SVG" in content[:4].upper():
        ext = "svg"
    return ext



import os
from pathlib import Path

import pytest

from reptor.lib.plugins.TestCaseToolPlugin import TestCaseToolPlugin

from ..Toolbase import MYMODULENAME


class TestMyModule(TestCaseToolPlugin):
    templates_path = os.path.normpath(Path(os.path.dirname(__file__)) / "../templates")

    @pytest.fixture(autouse=True)
    def setUp(self) -> None:
        MYMODULENAME.setup_class(
            Path(os.path.dirname(self.templates_path)), skip_user_plugins=True
        )
        self.mymodule = MYMODULENAME(reptor=self.reptor)

    def _load_xml_data(self, xml_file):
        self.mymodule.input_format = "xml"
        filepath = os.path.join(os.path.dirname(__file__), f"./data/{xml_file}")
        with open(filepath, "r") as f:
            self.mymodule.raw_input = f.read()

    def _load_json_data(self, json_file):
        self.mymodule.input_format = "json"
        filepath = os.path.join(os.path.dirname(__file__), f"./data/{json_file}")
        with open(filepath, "r") as f:
            self.mymodule.raw_input = f.read()

    def test_true_example(self):
        assert True == True
        raise NotImplementedError("Test not implemented")



import pathlib
import json
import toml
import yaml

from reptor.lib.plugins.Base import Base
from reptor.utils.table import make_table


class Projects(Base):
    """
    This plugin is used to interact with the projects via the sysreptor API.
    """

    meta = {
        "name": "Projects",
        "summary": "Queries Projects from reptor.api",
    }

    def __init__(self, **kwargs):
        super().__init__(**kwargs)
        self.search = kwargs.get("search")
        self.export = kwargs.get("export")
        self.duplicate = kwargs.get("duplicate")

    @classmethod
    def add_arguments(cls, parser, plugin_filepath=None):
        super().add_arguments(parser, plugin_filepath)
        project_parser = parser.add_mutually_exclusive_group()
        project_parser.add_argument(
            "-search",
            "--search",
            metavar="SEARCHTERM",
            help="Search for term",
            action="store",
            default=None,
        )
        project_parser.add_argument(
            "-export",
            "--export",
            help="Export project",
            choices=["archive", "json", "toml", "yaml"],
            type=str.lower,
            action="store",
            dest="export",
            default=None,
        )
        project_parser.add_argument(
            "-duplicate",
            "--duplicate",
            help="Duplicate project",
            action="store_true",
            dest="duplicate",
        )

    def _export_project(self, format="archive"):
        if format == "archive":
            filepath = pathlib.Path().cwd()
            file_name = filepath / f"{self.reptor.api.projects.project_id}.tar.gz"
            self.reptor.api.projects.export(file_name=file_name)
            return

        # Get Project
        project = self.reptor.api.projects.project.to_dict()
        if format == "json":
            self.console.print(json.dumps(project, indent=2))
        elif format == "toml":
            self.console.print(toml.dumps(project))
        elif format == "yaml":
            self.console.print(yaml.dump(project))

    def _search_project(self):
        if self.search is not None:
            projects = self.reptor.api.projects.search(self.search)
        else:
            projects = self.reptor.api.projects.get_projects()

        table = make_table(["Title", "ID", "Archived"])

        for project in projects:
            archived = ""
            if project.readonly:
                archived = "[red]Yes[/red]"
            table.add_row(project.name, project.id, archived)

        self.console.print(table)

    def _duplicate_project(self):
        duplicated_project = self.reptor.api.projects.duplicate()
        project_title = duplicated_project.name
        project_id = duplicated_project.id
        self.success(f"Duplicated to '{project_title}' ({project_id})")

    def run(self):
        if self.export:
            self._export_project(self.export)
        elif self.duplicate:
            self._duplicate_project()
        else:
            self._search_project()


loader = Projects



from functools import cached_property
from posixpath import join as urljoin

from reptor.api.APIClient import APIClient
from reptor.models.ProjectDesign import ProjectDesign


class ProjectDesignsAPI(APIClient):
    def __init__(self, **kwargs) -> None:
        super().__init__(**kwargs)

        self.base_endpoint = (
            f"{self.reptor.get_config().get_server()}/api/v1/projecttypes/"
        )

        self.project_design_id = kwargs.get("project_design_id", "")
        self.object_endpoint = urljoin(self.base_endpoint, self.project_design_id)

    @cached_property
    def project_design(self) -> ProjectDesign:
        """Gets project design

        Args:
            readonly (bool, optional): Only archived projects. Defaults to False.

        Returns:
            ProjectDesign object
        """
        if not self.project_design_id:
            raise ValueError("Missing Project Design ID")

        response = self.get(self.object_endpoint)
        return ProjectDesign(response.json())



import sys

from reptor.lib.plugins.UploadBase import UploadBase
from reptor.utils.table import make_table


class Note(UploadBase):
    """ """

    meta = {
        "name": "Note",
        "summary": "Uploads and lists a notes",
    }

    def __init__(self, **kwargs):
        super().__init__(**kwargs)
        self.list = kwargs.get("list")
        self.notename = self.reptor.get_config().get_cli_overwrite().get("notename")

    @classmethod
    def add_arguments(cls, parser, plugin_filepath=None):
        super().add_arguments(parser, plugin_filepath)
        parser.add_argument("--list", help="list available notes", action="store_true")

    def run(self):
        if self.list:
            self._list()
        else:
            self._write_note()

    def _list(self):
        notes = self.reptor.api.notes.get_notes()

        table = make_table(["Title", "ID"])
        for note in notes:
            table.add_row(note.title, note.id)
        self.console.print(table)

    def _write_note(self):
        parent_notename = None
        icon = None
        if self.notename:
            parent_notename = "Uploads"
        else:
            self.notename = "Uploads"
            icon = "📤"
        force_unlock = self.reptor.get_config().get_cli_overwrite().get("force_unlock")
        no_timestamp = self.reptor.get_config().get_cli_overwrite().get("no_timestamp")

        """
        Notes accept stdin only
        Notes are added as child of 'Uploads' note
        If no notename defined, content gets appended to 'Uploads' note
        """
        self.info("Reading from stdin...")
        content = sys.stdin.read()

        self.reptor.api.notes.write_note(
            content=content,
            notename=self.notename,
            parent_notename=parent_notename,
            icon=icon,
            force_unlock=force_unlock,  # type: ignore
            no_timestamp=no_timestamp,  # type: ignore
        )
        self.log.success("Successfully uploaded to notes.")


loader = Note



from reptor.lib.plugins.Base import Base
from reptor.utils.table import make_table


class Templates(Base):
    """
    Work with the finding templates.
    """

    meta = {
        "name": "Templates",
        "summary": "Queries Finding Templates from reptor.api",
    }

    def __init__(self, **kwargs):
        super().__init__(**kwargs)
        self.arg_search = kwargs.get("search")

    @classmethod
    def add_arguments(cls, parser, plugin_filepath=None):
        super().add_arguments(parser, plugin_filepath)
        templates_parsers = parser.add_argument_group()
        templates_parsers.add_argument(
            "--search", help="Search for term", action="store", default=None
        )

    def run(self):
        if not self.arg_search:
            templates = self.reptor.api.templates.get_templates()
        else:
            templates = self.reptor.api.templates.search(self.arg_search)

        table = make_table(["Title", "Usage Count", "Tags", "Translations", "ID"])
        for template in templates:
            main_translation = [t for t in template.translations if t.is_main][0]
            table.add_row(
                main_translation.data.title,
                str(template.usage_count),
                ",".join(template.tags),
                ",".join(t.language for t in template.translations),
                template.id,
            )

        self.console.print(table)


loader = Templates



import pathlib
import typing
from functools import cached_property
from posixpath import join as urljoin
from typing import Optional
import sys

from reptor.api.APIClient import APIClient
from reptor.models.Finding import FindingRaw
from reptor.models.Project import Project
from reptor.models.Section import Section, SectionRaw


class ProjectsAPI(APIClient):
    def __init__(self, **kwargs) -> None:
        super().__init__(**kwargs)
        self.project_design = None

        if not (server := self.reptor.get_config().get_server()):
            raise ValueError("No SysReptor server configured. Try 'reptor conf'.")
        if not self.project_id:
            raise ValueError(
                "No Project ID configured. Try 'reptor conf' or use '--project-id'."
            )

        self.base_endpoint = f"{server}/api/v1/pentestprojects"

        self.object_endpoint = urljoin(self.base_endpoint, self.project_id)
        self.debug(self.base_endpoint)

    def get_projects(self, readonly: bool = False) -> typing.List[Project]:
        """Gets list of projects

        Args:
            readonly (bool, optional): Only archived projects. Defaults to False.

        Returns:
            json: List of all Projects
        """
        url = self.base_endpoint
        if readonly:
            url = f"{url}?readonly=true"
        response = self.get(url)
        return_data = list()
        for item in response.json()["results"]:
            return_data.append(Project(item))
        return return_data

    def search(self, search_term: Optional[str] = "") -> typing.List[Project]:
        """Searches projects by search term and retrieves all projects that match

        Args:
            search_term (Optional[str], optional): Search Term to look for. Defaults to None.

        Returns:
            typing.List[Project]: List of projects that match search
        """

        response = self.get(f"{self.base_endpoint}?search={search_term}")

        return_data = list()
        for item in response.json()["results"]:
            return_data.append(Project(item))
        return return_data

    @cached_property
    def project(self) -> Project:
        return self._get_project()

    def _get_project(self) -> Project:
        if not self.project_id:
            raise ValueError("Make sure you have a project specified.")
        url = self.object_endpoint
        response = self.get(url)
        return Project(response.json())

    def export(self, file_name: typing.Optional[pathlib.Path] = None):
        """Exports a Project to a .tar.gz file locally.

        Args:
            file_name (typing.Optional[pathlib.Path], optional): Local File path. Defaults to stdout.

        Raises:
            ValueError: Requires project_id
        """
        if not self.project_id:
            raise ValueError(
                "No project ID. Specify in reptor conf or via -p / --project-id"
            )

        url = urljoin(self.base_endpoint, f"{self.project_id}/export/all")
        data = self.post(url)
        if file_name:
            with open(file_name, "wb") as f:
                f.write(data.content)
        else:
            sys.stdout.buffer.write(data.content)

    def duplicate(self) -> Project:
        """Duplicates Projects

        Returns:
            Project: Project Object
        """
        url = urljoin(self.base_endpoint, f"{self.project_id}/copy/")
        duplicated_project = self.post(url).json()
        return Project(duplicated_project)

    def get_sections(self) -> typing.List[Section]:
        """Gets all sections of a project

        Returns:
            typing.List[Section]: List of sections for this project
        """
        return_data = list()
        url = urljoin(self.base_endpoint, f"{self.project_id}/sections/")
        response = self.get(url).json()

        if not response:
            return return_data

        if not self.project_design:
            self.project_design = self.reptor.api.project_designs.project_design

        for item in response:
            section = Section(SectionRaw(item), self.project_design)
            return_data.append(section)
        return return_data

    def get_findings(self) -> typing.List[FindingRaw]:
        """Gets all findings of a project

        Returns:
            typing.List[FindingRaw]: List of findings for this project
        """
        url = urljoin(self.base_endpoint, f"{self.project_id}/findings/")
        response = self.get(url).json()

        if not response:
            return []
        return [FindingRaw(f) for f in response]

    def update_finding(self, finding_id: str, data: dict) -> FindingRaw:
        url = urljoin(self.base_endpoint, f"{self.project_id}/findings/{finding_id}/")
        return FindingRaw(self.patch(url, data).json())

    def create_finding(self, data: dict) -> FindingRaw:
        url = urljoin(self.base_endpoint, f"{self.project_id}/findings/")
        return FindingRaw(self.post(url, data).json())

    def create_finding_from_template(self, template_id: str) -> FindingRaw:
        url = urljoin(self.base_endpoint, f"{self.project_id}/findings/fromtemplate/")
        return FindingRaw(self.post(url, {"template": template_id}).json())

    def update_section(self, section_id: str, data: dict) -> SectionRaw:
        url = urljoin(self.base_endpoint, f"{self.project_id}/sections/{section_id}/")
        return SectionRaw(self.patch(url, data).json())

    def update_project(self, data: dict) -> Project:
        url = urljoin(self.base_endpoint, f"{self.project_id}/")
        return Project(self.patch(url, data).json())

    def get_enabled_language_codes(
        self,
    ) -> list:  # TODO should not be in ProjectsAPI probably
        url = urljoin(self.reptor.get_config().get_server(), "api/v1/utils/settings/")
        settings = self.get(url).json()
        languages = [
            l["code"] for l in settings.get("languages", list()) if l["enabled"] == True
        ]
        return languages



import pathlib

BASE_DIR: pathlib.Path = pathlib.Path(__file__).parent

PERSONAL_SYSREPTOR_HOME: pathlib.Path = pathlib.Path.home() / ".sysreptor"
PERSONAL_CONFIG_FILE: pathlib.Path = PERSONAL_SYSREPTOR_HOME / "config.yaml"

PLUGIN_DIRS: pathlib.Path = BASE_DIR / "plugins"

PLUGIN_DIRS_CORE: pathlib.Path = PLUGIN_DIRS / "core"
PLUGIN_DIRS_TOOLS: pathlib.Path = PLUGIN_DIRS / "tools"
PLUGIN_DIRS_PROJECTS: pathlib.Path = PLUGIN_DIRS / "projects"
PLUGIN_DIRS_UPLOADS: pathlib.Path = PLUGIN_DIRS / "uploads"
PLUGIN_DIRS_IMPORTERS: pathlib.Path = PLUGIN_DIRS / "importers"
PLUGIN_DIRS_EXPORTERS: pathlib.Path = PLUGIN_DIRS / "exporters"
PLUGIN_DIRS_USER: pathlib.Path = PERSONAL_SYSREPTOR_HOME / "plugins"

PLUGIN_IMPORT_DIRS = [
    PLUGIN_DIRS_CORE,
    PLUGIN_DIRS_TOOLS,
    PLUGIN_DIRS_PROJECTS,
    PLUGIN_DIRS_UPLOADS,
    PLUGIN_DIRS_IMPORTERS,
    PLUGIN_DIRS_EXPORTERS,
    PLUGIN_DIRS_USER,  # Should be last to override other plugins
]

PLUGIN_TEMPLATES_DIR_NAME: str = "templates"
FINDING_TEMPLATES_DIR_NAME: str = "findings"

PLUGIN_TOOLBASE_TEMPLATE_FOLDER: pathlib.Path = BASE_DIR / "templates" / "Toolbase"

LOG_FOLDER: pathlib.Path = PERSONAL_SYSREPTOR_HOME / "logs"

NEWLINE = "\n"

USER_AGENT = "reptor CLI v0.1.0"  # TODO dynamic version


# Django Related Setup
TEMPLATES = [
    {
        "BACKEND": "django.template.backends.django.DjangoTemplates",
        "DIRS": [],
        "OPTIONS": {
            "libraries": {
                "md": "reptor.utils.templatetags.md",
            },
        },
    },
]

DEBUG = True
INSTALLED_APPS = []
LOGGING_CONFIG = []
LOGGING = []
FORCE_SCRIPT_NAME = ""

DEFAULT_PROJECT_DESIGN = {
    "report_fields": {
        "title": {
            "type": "string",
            "label": "Title",
            "origin": "core",
            "default": "TODO: Report Title",
            "required": True,
            "spellcheck": True,
        }
    },
    "report_sections": [{"id": "other", "label": "Other", "fields": ["title"]}],
    "finding_fields": {
        "cvss": {
            "type": "cvss",
            "label": "CVSS",
            "origin": "predefined",
            "default": "n/a",
            "required": True,
        },
        "title": {
            "type": "string",
            "label": "Title",
            "origin": "core",
            "default": "TODO: Finding Title",
            "required": True,
            "spellcheck": True,
        },
        "impact": {
            "type": "markdown",
            "label": "Impact",
            "origin": "predefined",
            "default": "TODO: impact of finding",
            "required": True,
        },
        "summary": {
            "type": "markdown",
            "label": "Summary",
            "origin": "predefined",
            "default": "TODO: High-level summary",
            "required": True,
        },
        "severity": {
            "type": "enum",
            "label": "Severity",
            "origin": "predefined",
            "choices": [
                {"label": "Info", "value": "info"},
                {"label": "Low", "value": "low"},
                {"label": "Medium", "value": "medium"},
                {"label": "High", "value": "high"},
                {"label": "Critical", "value": "critical"},
            ],
            "default": None,
            "required": True,
        },
        "references": {
            "type": "list",
            "items": {
                "type": "string",
                "label": "Reference",
                "origin": "predefined",
                "default": None,
                "required": True,
                "spellcheck": False,
            },
            "label": "References",
            "origin": "predefined",
            "required": False,
        },
        "description": {
            "type": "markdown",
            "label": "Technical Description",
            "origin": "predefined",
            "default": "TODO: detailed technical description what this findings is about and how it can be exploited",
            "required": True,
        },
        "precondition": {
            "type": "string",
            "label": "Precondition",
            "origin": "predefined",
            "default": None,
            "required": True,
            "spellcheck": True,
        },
        "retest_notes": {
            "type": "markdown",
            "label": "Re-test Notes",
            "origin": "predefined",
            "default": None,
            "required": False,
        },
        "retest_status": {
            "type": "enum",
            "label": "Re-test Status",
            "origin": "predefined",
            "choices": [
                {"label": "Open", "value": "open"},
                {"label": "Resolved", "value": "resolved"},
                {"label": "Partially Resolved", "value": "partial"},
                {"label": "Changed", "value": "changed"},
                {"label": "Accepted", "value": "accepted"},
                {"label": "New", "value": "new"},
            ],
            "default": None,
            "required": False,
        },
        "wstg_category": {
            "type": "enum",
            "label": "OWASP Web Security Testing Guide Category",
            "origin": "predefined",
            "choices": [
                {"label": "INFO - Information Gathering", "value": "INFO"},
                {
                    "label": "CONF - Configuration and Deployment Management",
                    "value": "CONF",
                },
                {"label": "IDNT - Identity Management", "value": "IDNT"},
                {"label": "ATHN - Authentication", "value": "ATHN"},
                {"label": "ATHZ - Authorization", "value": "ATHZ"},
                {"label": "SESS - Session Management", "value": "SESS"},
                {"label": "INPV - Input Validation", "value": "INPV"},
                {"label": "ERRH - Error Handling", "value": "ERRH"},
                {"label": "CRYP - Weak Cryptography", "value": "CRYP"},
                {"label": "BUSL - Business Logic", "value": "BUSL"},
                {"label": "CLNT - Client-side Testing", "value": "CLNT"},
                {"label": "APIT - API Testing", "value": "APIT"},
            ],
            "default": None,
            "required": True,
        },
        "recommendation": {
            "type": "markdown",
            "label": "Recommendation",
            "origin": "predefined",
            "default": "TODO: how to fix the vulnerability",
            "required": True,
        },
        "owasp_top10_2021": {
            "type": "enum",
            "label": "OWASP Top 10 - 2021",
            "origin": "predefined",
            "choices": [
                {"label": "A01:2021 - Broken Access Control", "value": "A01_2021"},
                {"label": "A02:2021 - Cryptographic Failures", "value": "A02_2021"},
                {"label": "A03:2021 - Injection", "value": "A03_2021"},
                {"label": "A04:2021 - Insecure Design", "value": "A04_2021"},
                {"label": "A05:2021 - Security Misconfiguration", "value": "A05_2021"},
                {
                    "label": "A06:2021 - Vulnerable and Outdated Components",
                    "value": "A06_2021",
                },
                {
                    "label": "A07:2021 - Identification and Authentication Failures",
                    "value": "A07_2021",
                },
                {
                    "label": "A08:2021 - Software and Data Integrity Failures",
                    "value": "A08_2021",
                },
                {
                    "label": "A09:2021 - Security Logging and Monitoring Failures",
                    "value": "A09_2021",
                },
                {
                    "label": "A10:2021 - Server-Side Request Forgery (SSRF)",
                    "value": "A10_2021",
                },
            ],
            "default": None,
            "required": True,
        },
        "affected_components": {
            "type": "list",
            "items": {
                "type": "string",
                "label": "Component",
                "origin": "predefined",
                "default": "TODO: affected component",
                "required": True,
                "spellcheck": False,
            },
            "label": "Affected Components",
            "origin": "predefined",
            "required": True,
        },
        "short_recommendation": {
            "type": "string",
            "label": "Short Recommendation",
            "origin": "predefined",
            "default": "TODO: short recommendation",
            "required": True,
            "spellcheck": False,
        },
    },
}



import typing

from reptor.lib.plugins.Base import Base
from reptor.models.FindingTemplate import FindingTemplate


class BaseImporter(Base):
    reptor: typing.Any
    mapping: typing.Dict
    meta: typing.Dict = {
        "name": "",
        "author": "",
        "version": "",
        "website": "",
        "license": "",
        "tags": [],
        "summary": "",
    }

    def __init__(self, **kwargs):
        super().__init__(**kwargs)
        self.reptor = kwargs.get("reptor", None)
        self.finding_language = kwargs.get("language", "en-US")
        self.tags = list(filter(None, (kwargs.get("tags") or "").split(",")))
        self.tags.append(f"{self.__class__.__name__.lower()}:imported")

    @classmethod
    def add_arguments(cls, parser, plugin_filepath=None):
        action_group = parser.add_argument_group()
        action_group.title = "Global Importer Settings"
        action_group.add_argument(
            "-tags",
            "--tags",
            action="store",
            help="Comma-separated tags for new templates",
        )

    def next_findings_batch(self) -> typing.List[typing.Dict]:
        """Implement this to yield the next findings to process"""
        raise NotImplementedError(
            "next_findings_batch not implemented in importer plugin"
        )

    def _create_finding_item(
        self,
        finding_data: typing.Union[typing.Dict, typing.List],
        tags: typing.List[str] = None,
    ) -> FindingTemplate:
        if not tags:
            tags = []

        if isinstance(finding_data, dict):
            finding_data = [finding_data]

        translations = list()
        for translation in finding_data:
            mapped_data = dict()
            for key, value in self.mapping.items():
                try:
                    converted_data = translation["data"][key]
                except KeyError:
                    continue
                # check if we have a convert_method and call it
                # update the value
                convert_method_name = f"convert_{key}"
                if hasattr(self, convert_method_name):
                    if callable(getattr(self, convert_method_name)):
                        converter_method = getattr(self, convert_method_name)
                        self.debug(f"Calling: {convert_method_name}")
                        converted_data = converter_method(translation["data"][key])

                # get type of converted data. the real type. not data type "type"
                mapped_data.setdefault(value, type(converted_data)())
                if isinstance(converted_data, str):
                    if mapped_data[value]:
                        mapped_data[value] += "\n"
                    mapped_data[value] += converted_data
                elif isinstance(converted_data, dict):
                    mapped_data.update(converted_data)
                else:
                    mapped_data[value] += converted_data

            translation["data"] = mapped_data
            translations.append(translation)

        new_finding = FindingTemplate({"tags": tags, "translations": translations})
        return new_finding

    def _upload_finding_template(self, new_finding: FindingTemplate):
        updated_template = self.reptor.api.templates.upload_new_template(new_finding)
        if updated_template:
            self.success(f'Successfully uploaded "{updated_template.id}"')
        else:
            self.error("Cancel? [Y/n]")
            abort_answer = input()[:1].lower()
            if abort_answer != "n":
                raise AssertionError("Cancelling")

    def run(self):
        try:
            self.mapping
        except AttributeError as e:
            raise AttributeError("Importer plugin does not define a mapping") from e

        for external_finding in self.next_findings_batch():  # type: ignore
            new_finding = self._create_finding_item(external_finding)
            if not new_finding:
                continue
            self.display(f'Uploading "{new_finding.translations[0].data.title}"')
            self.debug(new_finding)
            self._upload_finding_template(new_finding)



import typing
from copy import deepcopy

from reptor.models.Base import BaseModel, FindingTemplateSources
from reptor.models.Finding import FindingDataRaw


class FindingTemplateTranslation(BaseModel):
    language: str = "en-US"
    status: str = "in-progress"
    is_main: bool = True
    data: FindingDataRaw

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        # Init mandatory fields
        self.is_main = True

    def to_dict(self) -> dict:
        result = deepcopy(vars(self))
        result["data"] = self.data.to_dict()
        return result


class FindingTemplate(BaseModel):
    """
    Attributes:
        details:
        images:
        lock_info:
        usage_count:
        source:
        tags:
        translations:
    """

    details: str = ""
    images: str = ""
    lock_info: bool = False
    usage_count: int = 0
    source: FindingTemplateSources = FindingTemplateSources.CREATED
    tags: typing.List[str] = []
    translations: typing.List[FindingTemplateTranslation] = []

    def to_dict(self) -> dict:
        result = deepcopy(vars(self))
        if isinstance(self.source, FindingTemplateSources):
            result["source"] = self.source.value
        result["translations"] = [t.to_dict() for t in self.translations]
        return result



import importlib
import os
import pathlib
import sys
import typing

import reptor.settings as settings
import reptor.subcommands as subcommands
from reptor.lib.plugins.DocParser import DocParser


class PluginManager:
    """The  PluginManager is responsible for loading all plugins and making
    them available to reptor.
    """

    _plugin_paths: typing.List = list()

    LOADED_PLUGINS: typing.Dict = dict()

    _reptor: typing.Any

    def __init__(self, reptor) -> None:
        self._reptor = reptor

    def run_loading_sequence(self):
        self._reptor.logger.info("Loading plugins...")
        self._load_modules()

    def is_loaded(self, plugin_name: str) -> bool:
        """Checks if a plugin is loaded

        Args:
            plugin_name (str): Plugin Name

        Returns:
            bool: True if found, False if not
        """
        if plugin_name in self.LOADED_PLUGINS.keys():
            return True
        return False

    def unload(self, plugin_name: str) -> bool:
        """Unloads a plugin by its name

        Args:
            plugin_name (str): Plugin Name

        Returns:
             bool: True if unloaded, otherwise False
        """
        if self.is_loaded(plugin_name):
            del self.LOADED_PLUGINS[plugin_name]
            return True
        return False

    def get_plugin_by_name(self, plugin_name: str):
        """Returns a plugin by its name, if it exists
        otherwise return is None

        Args:
            plugin_name (str): Plugin Name
        """
        if self.is_loaded(plugin_name):
            return self.LOADED_PLUGINS[plugin_name]
        return None

    def _load_plugin_from_path(self, directory: pathlib.Path):
        """Loads a File and Folder based Modules from the ./modules folder in reptor

        Returns:
            typing.List: Holds absolute paths to each Module file
        """
        self._reptor.logger.debug(f"Loading Modules from folder: {directory}")
        plugin_paths = list()
        for modules_path in directory.glob("*"):
            if "__pycache__" in modules_path.name:
                continue
            if modules_path.is_dir():
                for modules_file in modules_path.glob("*.py"):
                    plugin_paths.append(str(modules_file))
            elif modules_path.suffix == ".py":
                plugin_paths.append(str(modules_path))

        self._plugin_paths += plugin_paths

        self._reptor.logger.debug(f"Found Plugin Paths: {plugin_paths}")

    def _load_modules(self):
        """Loads the core modules for reptor to function"""
        for plugin_path in settings.PLUGIN_IMPORT_DIRS:
            self._load_plugin_from_path(plugin_path)

    def load_plugins(self):
        """Loads each plugin from _plugin_paths

        Returns:
            typing.Dict: Dictionary holding each plugin meta information
        """

        # Be careful here: We are working technically with python modules, but
        # in reptor we call our different implementations
        # for various tools: plugins

        for plugin_path in self._plugin_paths:
            spec = importlib.util.spec_from_file_location(  # type: ignore
                "module.name", plugin_path
            )

            # self._reptor.logger.debug(plugin_path)

            module = importlib.util.module_from_spec(spec)  # type: ignore

            sys.modules["module.name"] = module
            spec.loader.exec_module(module)

            # Check if the Module is Valid
            if not hasattr(module, "loader"):
                continue

            # Configure metadata
            plugin_docs = DocParser.parse(module.loader.meta)
            plugin_docs.name = module.loader.__name__.lower()
            plugin_docs.path = plugin_path

            # Check what type of module it is and mark it as such
            if str(settings.PLUGIN_DIRS_USER) in plugin_path:
                plugin_docs.category = "private"
            else:
                plugin_docs.category = os.path.basename(plugin_path)

            # Add it to the correct commands group
            subcommands_idx = (
                module.loader.__base__
                if module.loader.__base__ in subcommands.SUBCOMMANDS_GROUPS
                else "other"
            )

            for index, existing_module in enumerate(
                subcommands.SUBCOMMANDS_GROUPS[subcommands_idx][1]
            ):
                # check if the module is already in the list,
                # if so a later loaded module (e.g. private plugin from home dir)
                # needs to overwrite it
                if existing_module.name == plugin_docs.name:
                    # we have the case of an overwrite
                    # save the overwritten module details
                    plugin_docs.set_overwrites_plugin(existing_module)

                    # remove the original item
                    subcommands.SUBCOMMANDS_GROUPS[subcommands_idx][1].pop(index)
            # add the module data to the end
            subcommands.SUBCOMMANDS_GROUPS[subcommands_idx][1].append(plugin_docs)

            # because it is a dictionary, an overwritten module is automatically overwritten
            self.LOADED_PLUGINS[plugin_docs.name] = module

        self._reptor.logger.debug(f"Loaded Plugins are: {self.LOADED_PLUGINS}")



import sys
import typing

from reptor.lib.console import reptor_console


class Base:
    """
    Provides all bases with access to the Reptor app.

    Attributes:
        reptor: Reptor Instance
        notename: The Notename to be used when uploading
    """

    reptor: typing.Any
    meta: typing.Dict = {
        "name": "",
        "author": "",
        "version": "",
        "website": "",
        "license": "",
        "tags": [],
        "summary": "",
    }

    keys: typing.Dict = {}

    def __init__(self, **kwargs):
        self.notename = kwargs.get("notename")
        self.file_path = kwargs.get("file", "")
        self.reptor = kwargs.get("reptor", None)

        plugin_name = self.__class__.__name__.lower()
        for plugin_config_key in self.reptor.get_config().get_config_keys(
            plugin=plugin_name
        ):
            if not hasattr(self, plugin_config_key):
                self.__setattr__(
                    plugin_config_key,
                    self.reptor.get_config().get(plugin_config_key, plugin=plugin_name),
                )

        self._check_required_keys_are_set(plugin_name)

    def _check_required_keys_are_set(self, plugin_name: str = ""):
        """This method will check if the keys specified in the attribute keys
        are present in the config. If they are not present in the config, it will
        over the user to set them.

        The form of each key should be key:help , i.e:

        ```
        {
            "api_secret": "Used to connect to the API",
            "language": "default language of the source items"
        }
        ```
        """
        config_updated = False
        for key in self.keys:
            config_value = self.reptor.get_config().get(key, "", plugin=plugin_name)
            if not config_value:
                self.console.print(
                    f"[red]Required Key {key} not set in config. [bold]Set it now to continue or press ENTER to exit:[/bold][/red]"
                )
                config_value = input()
                if not config_value:
                    self.console.print("[green]Exiting...[/green]")
                    sys.exit(0)
                self.reptor.get_config().set(key, config_value, plugin=plugin_name)
                config_updated = True

        if config_updated:
            self.reptor.get_config().store_config()

    @classmethod
    def add_arguments(cls, parser, plugin_filepath=None):
        pass

    @property
    def log(self):
        """Access the logging directly from plugin

        Returns:
            ReptorAdapter: Logging Module
        """
        return self.reptor.get_logger()

    @property
    def console(self):
        """Access the rich console that allows markdown etc.

        Returns:
            Console: rich console
        """
        return reptor_console

    def success(self, msg, *args, **kwargs):
        """Use this to print Green text by default. You can change colors etc.

        See the logger.py for examples.

        Args:
            msg (str): Any message you want to print
        """
        self.log.success(msg, *args, **kwargs)

    def display(self, msg, *args, **kwargs):
        """Use this to print blue text by default. You can change colors etc.

        See the logger.py for examples.

        Args:
            msg (str): Any message you want to print
        """
        self.log.display(msg, *args, **kwargs)

    def highlight(self, msg, *args, **kwargs):
        """Prints a yellow message. Good for highlighting certain
        output.

        Args:
            msg (str): Any message you want to print
        """
        self.log.highlight(msg, *args, **kwargs)

    def debug(self, msg, *args, **kwargs):
        """Default DEBUG method of the logger. Use this instead of accessing
        log or reptor.get_logger()

        Args:
            msg (str): Message to show in DEBUG log
        """
        self.log.debug(msg, *args, **kwargs)

    def info(self, msg, *args, **kwargs):
        """Default INFO method of the logger. Use this instead of accessing
        log or reptor.get_logger()

        Args:
            msg (str): Message to show in INFO log
        """
        self.log.info(msg, *args, **kwargs)

    def warning(self, msg, *args, **kwargs):
        """Default WARNING method of the logger. Use this instead of accessing
        log or reptor.get_logger()

        Args:
            msg (str): Message to show in WARNING log
        """
        self.log.warning(msg, *args, **kwargs)

    def error(self, msg, *args, **kwargs):
        """Default ERROR method of the logger. Use this instead of accessing
        log or reptor.get_logger()

        Args:
            msg (str): Message to show in ERROR log
        """
        self.log.error(msg, *args, **kwargs)

    def run(self):
        pass



from reptor.lib.plugins.ToolBase import ToolBase
from reptor.plugins.tools.Nikto.models import (
    NiktoScan,
    ScanDetails,
    Item,
    Statistics,
)


class Nikto(ToolBase):
    """
    cat nikto-raw-output.txt | reptor simplelist -c format

    cat nikto-result.xml | python reptor simplelist --xml

    """

    meta = {
        "author": "Richard Schwabe",
        "name": "Nikto",
        "version": "1.1",
        "license": "MIT",
        "tags": ["web", "owasp"],
        "summary": "Formats Nikto output (XML)",
    }

    keys = {}

    def __init__(self, **kwargs):
        super().__init__(**kwargs)
        self.notename = kwargs.get("notename", "Nikto")

    def parse_xml(self):
        """Parses XML file from Nikto, tested with version 2.5.0

        Args:
            data (str): Raw String Input of XML

        Returns:
            typing.List[NiktoScan]: Returns a list of NiktoScan objects
        """
        super().parse_xml(as_dict=False)
        self.debug("Running parse_xml of nikto")

        return_data = list()
        for niktoscan in self.xml_root:
            self.debug(f"Got niktoscan: {niktoscan} ")
            nikto = NiktoScan()
            nikto.parse(niktoscan)

            scandetails = ScanDetails()
            scandetails.parse(niktoscan[0])

            statistics = Statistics()
            statistics.parse(niktoscan[0].find("statistics"))

            for item_data in niktoscan[0].iter("item"):
                item = Item()
                item.parse(item_data)
                scandetails.append_item(item)

            nikto.statistics = statistics
            nikto.scandetails = scandetails

            return_data.append(nikto)

        self.parsed_input = return_data


loader = Nikto



from django.utils.html import strip_tags

from reptor.lib.importers.BaseImporter import BaseImporter

try:
    from gql import Client, gql
    from gql.transport.aiohttp import AIOHTTPTransport
    from gql.transport.exceptions import TransportServerError
except ImportError:
    gql = None


class GhostWriter(BaseImporter):
    """
    Imports findings from GhostWriter

    Connects to the GraqhQL API of a GhostWriter instance and imports its
    finding templates to SysReptor via API.
    """

    meta = {
        "author": "Richard Schwabe",
        "name": "GhostWriter",
        "version": "1.0",
        "license": "MIT",
        "summary": "Imports GhostWriter finding templates",
    }

    mapping = {
        "title": "title",
        "cvss_vector": "cvss",
        "description": "summary",
        "findingGuidance": "description",
        "replication_steps": "description",
        "hostDetectionTechniques": "description",
        "networkDetectionTechniques": "description",
        "impact": "impact",
        "mitigation": "recommendation",
        "references": "references",
    }
    ghostwriter_url: str
    apikey: str

    def __init__(self, **kwargs) -> None:
        super().__init__(**kwargs)

        if gql is None:
            raise ImportError(
                "Error importing gql. Install with 'pip install reptor{self.log.escape('[ghostwriter]')}'."
            )

        self.ghostwriter_url = kwargs.get("url", "")
        if not self.ghostwriter_url:
            try:
                self.ghostwriter_url = self.url
            except AttributeError:
                raise ValueError("Ghostwriter URL is required.")
        if not hasattr(self, "apikey"):
            raise ValueError(
                "Ghostwriter API Key is required. Add to your user config."
            )
        self.insecure = kwargs.get("insecure", False)

    @classmethod
    def add_arguments(cls, parser, plugin_filepath=None):
        super().add_arguments(parser, plugin_filepath)
        action_group = parser.add_argument_group()
        action_group.add_argument(
            "-url",
            "--url",
            metavar="URL",
            action="store",
            const="",
            nargs="?",
            help="API Url",
        )

    def convert_references(self, value):
        value = strip_tags(value)
        return [l for l in value.splitlines() if l.strip()]

    def convert_hostDetectionTechniques(self, value):
        if strip_tags(value):
            return f"\n\n**Host Detection Techniques**\n\n{value}"
        return value

    def convert_networkDetectionTechniques(self, value):
        if strip_tags(value):
            return f"\n\n**Network Detection Techniques**\n\n{value}"
        return value

    def convert_findingGuidance(self, value):
        if strip_tags(value):
            return f"TODO: {value}\n\n"
        return value

    def _get_ghostwriter_findings(self):
        query = gql(
            """
            query MyQuery {
                finding {
                        findingGuidance
                        networkDetectionTechniques
                        hostDetectionTechniques
                        replication_steps
                        cvss_vector
                        description
                        impact
                        mitigation
                        references
                        title
                    }
            }
            """
        )

        # Either x-hasura-admin-secret or Authorization Bearer can be used
        headers = {
            "Content-Type": "application/json",
        }
        if (
            self.apikey.startswith("ey")
            and "." in self.apikey
            and len(self.apikey) > 40
        ):
            # Probably a JWT token
            headers["Authorization"] = f"Bearer {self.apikey}"
        else:
            # Probably hasura admin secret
            headers["x-hasura-admin-secret"] = self.apikey

        transport = AIOHTTPTransport(
            url=f"{self.ghostwriter_url}/v1/graphql", headers=headers
        )

        client = Client(transport=transport, fetch_schema_from_transport=False)
        try:
            result = client.execute(query)
        except TransportServerError as e:
            if e.code == 404:
                raise TransportServerError(
                    "404, Not found. Wrong URL? Make sure to specify the graqhql port (default: 8080)."
                ) from e
            raise e
        return result["finding"]

    def next_findings_batch(self):
        self.debug("Running batch findings")

        findings = self._get_ghostwriter_findings()
        for finding_data in findings:
            yield {
                "language": "en-US",
                "status": "in-progress",
                "data": finding_data,
            }


loader = GhostWriter



import pathlib
import typing

# Todo: Because we are not using the docs anymore, we should refactor this and rename this


class PluginDocs:
    _overwrites = None

    name: str = ""
    author: str = ""
    version: str = ""
    website: str = ""
    license: str = ""
    tags: list = []
    summary: str = ""
    category: str = ""

    path: pathlib.Path = None  # type: ignore

    def set_overwrites_plugin(self, plugin):
        self._overwrites = plugin

    def get_overwritten_plugin(self):
        return self._overwrites


class DocParser:
    @staticmethod
    def parse(meta_dictionary: typing.Dict) -> PluginDocs:
        plugin_docs = PluginDocs()
        plugin_docs.author = meta_dictionary.get("author", "")
        plugin_docs.version = meta_dictionary.get("version", "")
        plugin_docs.website = meta_dictionary.get("website", "")
        plugin_docs.license = meta_dictionary.get("license", "")
        plugin_docs.tags = meta_dictionary.get("tags", [])
        plugin_docs.summary = meta_dictionary.get("summary", "")

        return plugin_docs



import typing

import requests

import reptor.settings as settings
from reptor.lib.console import reptor_console


class APIClient:
    """Base API Client, holds all endpoint configuration and supplies subclasses with HTTP methods"""

    reptor: typing.Any
    base_endpoint: str
    endpoint: str
    item_id: str
    force_unlock: bool
    project_id: str

    def __init__(self, require_project_id=True, **kwargs) -> None:
        self.reptor = kwargs.get("reptor", None)
        if not self.reptor:
            reptor_console.print(
                "[red]Make sure you access the API via the apimanager (reptor.api)[/red]"
            )
            exit(1)
        self.verify = not self.reptor.get_config().get("insecure", False)

        try:
            self.project_id = self.reptor.get_active_project_id()
            self.debug(f"Project ID is {self.project_id}")
        except ValueError as e:
            if require_project_id:
                raise e
            self.project_id = ""

    def _get_headers(self, json_content=False) -> typing.Dict:
        headers = dict()
        if json_content:
            headers["Content-Type"] = "application/json"
        headers["User-Agent"] = settings.USER_AGENT
        headers["Authorization"] = f"Bearer {self.reptor.get_config().get_token()}"
        self.debug(f"HTTP Headers: {headers}")
        return headers

    @property
    def log(self):
        """Access the logging directly from plugin

        Returns:
            ReptorAdapter: Logging Module
        """
        return self.reptor.get_logger()

    @property
    def console(self):
        """Access the rich console that allows markdown etc.

        Returns:
            Console: rich console
        """
        return reptor_console

    def success(self, msg, *args, **kwargs):
        """Use this to print Green text by default. You can change colors etc.

        See the logger.py for examples.

        Args:
            msg (str): Any message you want to print
        """
        self.log.success(msg, *args, **kwargs)

    def display(self, msg, *args, **kwargs):
        """Use this to print blue text by default. You can change colors etc.

        See the logger.py for examples.

        Args:
            msg (str): Any message you want to print
        """
        self.log.display(msg, *args, **kwargs)

    def highlight(self, msg, *args, **kwargs):
        """Prints a yellow message. Good for highlighting certain
        output.

        Args:
            msg (str): Any message you want to print
        """
        self.log.highlight(msg, *args, **kwargs)

    def debug(self, msg, *args, **kwargs):
        """Default DEBUG method of the logger. Use this instead of accessing
        log or reptor.get_logger()

        Args:
            msg (str): Message to show in DEBUG log
        """
        self.log.debug(msg, *args, **kwargs)

    def info(self, msg, *args, **kwargs):
        """Default INFO method of the logger. Use this instead of accessing
        log or reptor.get_logger()

        Args:
            msg (str): Message to show in INFO log
        """
        self.log.info(msg, *args, **kwargs)

    def warning(self, msg, *args, **kwargs):
        """Default WARNING method of the logger. Use this instead of accessing
        log or reptor.get_logger()

        Args:
            msg (str): Message to show in WARNING log
        """
        self.log.warning(msg, *args, **kwargs)

    def error(self, msg, *args, **kwargs):
        """Default ERROR method of the logger. Use this instead of accessing
        log or reptor.get_logger()

        Args:
            msg (str): Message to show in ERROR log
        """
        self.log.error(msg, *args, **kwargs)

    def get(self, url: str) -> requests.models.Response:
        """Sends a get request

        Args:
            url (str): Endpoint URL

        Returns:
            requests.models.Response: Returns requests Response Object
        """
        self.debug(f"GET URL:{url}")
        response = requests.get(
            url,
            headers=self._get_headers(),
            verify=self.verify,
        )
        self.debug(f"Received response: {response.content}")
        response.raise_for_status()
        return response

    def post(
        self, url: str, data=None, files=None, json_content: bool = True
    ) -> requests.models.Response:
        """Sends a post requests, requires some json data

        Args:
            url (str): Endpoint URL
            data (_type_, optional): _description_. Defaults to None.
            files (_type_, optional): _description_. Defaults to None.
            json_content (bool, optional): _description_. Defaults to True.

        Returns:
            requests.models.Response: Requests Responde Object
        """

        self.reptor.logger.debug(f"POST URL: {url}")

        response = requests.post(
            url,
            headers=self._get_headers(json_content=json_content),
            json=data,
            files=files,
            verify=self.verify,
        )
        self.debug(f"Received response: {response.content}")
        response.raise_for_status()
        return response

    def put(self, url: str, data: object) -> requests.models.Response:
        """Sends a put requests, requires some json data

        Args:
            url (str): Endpoint URL
            data (object): JSON Data

        Returns:
            requests.models.Response: requests Respone Object
        """
        self.debug(f"PUT URL:{url}")
        response = requests.put(
            url,
            headers=self._get_headers(),
            json=data,
            verify=self.verify,
        )
        self.debug(f"Received response: {response.content}")
        response.raise_for_status()
        return response

    def patch(self, url: str, data: object) -> requests.models.Response:
        """Sends a patch requests, requires some json data

        Args:
            url (str): Endpoint URL
            data (object): JSON Data

        Returns:
            requests.models.Response: requests Respone Object
        """
        self.debug(f"PATCH URL:{url}")
        response = requests.patch(
            url,
            headers=self._get_headers(),
            json=data,
            verify=self.verify,
        )
        self.debug(f"Received response: {response.content}")
        response.raise_for_status()
        return response



import argparse
import logging
import signal
import sys
import traceback

import django
from django.conf import settings as django_settings

import reptor.settings as settings
import reptor.subcommands as subcommands
from reptor.api.manager import APIManager
from reptor.lib.conf import Config
from reptor.lib.console import Console, reptor_console
from reptor.lib.logger import ReptorAdapter, reptor_logger
from reptor.lib.pluginmanager import PluginManager
from reptor.lib.plugins.DocParser import PluginDocs
from reptor.utils.django_tags import setup_django_tags
from reptor.utils.markdown import convert_markdown_to_console

root_logger = logging.getLogger("root")


def signal_handler(sig, frame):
    reptor_console.print("[yellow]You pressed Ctrl+C![/yellow]")
    sys.exit(0)


class Reptor:
    """The reptor class is the main App.

    It is responsible for the correct loading of plugins via the PluginManager,
    enabling global arguments and forwarding any plugin calls to the correct
    plugin.

    Attributes:
        plugin_manager: AcAPIManagercess any plugins
        logger: Log to the logger
        console: Write to console via print
        api: Access the API via APIManager

    """

    _config: Config
    _parser: argparse.ArgumentParser
    _sub_parsers: argparse._SubParsersAction
    plugin_manager: PluginManager

    logger: ReptorAdapter = reptor_logger
    console: Console = reptor_console
    _api: APIManager

    def __new__(cls):
        if not hasattr(cls, "instance"):
            cls.instance = super(Reptor, cls).__new__(cls)
        return cls.instance

    def __init__(self) -> None:
        signal.signal(signal.SIGINT, signal_handler)

        # Load the config
        self._config = Config()
        self._config.load_config()

        # Setup Django tags
        setup_django_tags()

        self.plugin_manager = PluginManager(self)

        if self.get_config().get_log_file():
            self.logger.add_file_log()

    def get_config(self) -> Config:
        """Use this to access the current config

        Returns:
            Config: Current Configuration
        """
        return self._config

    def get_logger(self) -> ReptorAdapter:
        """Returns the active logger. Use this for logging

        Returns:
            ReptorAdapter: _description_
        """
        return self.logger

    def get_plugin_manager(self) -> PluginManager:
        """Returns the PluginManager with all loaded plugins.

        Returns:
            PluginManager: _description_
        """
        return self.plugin_manager

    def get_active_project_id(self) -> str:
        """Always returns the active project id, either from config
        or from the CLI overwrite. If no project id is found,
        then it returns an emptry string.

        Returns:
            str: Project ID
        """
        if self.get_config().get_cli_overwrite().get("project_id", ""):
            return self.get_config().get_cli_overwrite().get("project_id", "")
        if self.get_config().get_project_id():
            return self.get_config().get_project_id()
        return ""

    @property
    def api(self) -> APIManager:
        return self._api

    def _create_parsers(self):
        """Creates the description in the help and the parsers to be used

        Returns:
            parser,subparsers: ArgumentParser and SubParser
        """
        # Argument parser description
        description = ""
        for (
            _,
            short_help_group_meta,
        ) in subcommands.SUBCOMMANDS_GROUPS.items():
            description += f"\n{short_help_group_meta[0]}:\n"

            item: PluginDocs
            for item in short_help_group_meta[1]:
                description += f"{item.name:<21} {item.summary}{settings.NEWLINE}"

        # Argument parser
        main_description = """
            Examples:
                reptor conf
                echo "Upload this!" | reptor note
                reptor file data/*
                cat sslyze.json | reptor sslyze --json --push-findings
                cat nmap.xml | reptor nmap --xml --upload --multi-notes
"""
        self._parser = argparse.ArgumentParser(
            prog="reptor",
            description=main_description.strip(),
            formatter_class=argparse.RawDescriptionHelpFormatter,
        )

        self._sub_parsers = self._parser.add_subparsers(
            dest="command", description=description, help=argparse.SUPPRESS
        )

    def _dynamically_add_plugin_options(self):
        """
        Calls add_arguments on each plugin that is loaded and provides
        the subparser as well as the plugin file path
        """
        # Dynamically add plugin options
        for name, plugin in self.plugin_manager.LOADED_PLUGINS.items():
            plugin.subparser = self._sub_parsers.add_parser(
                name,
                description=plugin.loader.meta.get("summary", ""),
                formatter_class=argparse.RawTextHelpFormatter,
            )
            plugin.loader.add_arguments(
                plugin.subparser, plugin_filepath=plugin.__file__
            )

    def _add_config_parse_options(self):
        """Creates the configuration arguments

        Args:
            parser (argparse.ArgumentParser): main parser
        """
        config_parser = self._parser.add_argument_group("configuration")
        config_parser.add_argument("-s", "--server")
        config_parser.add_argument("-t", "--token", help="SysReptor API token")
        config_parser.add_argument(
            "--insecure", help="do not verify server certificate", action="store_true"
        )
        config_parser.add_argument("-p", "--project-id", help="SysReptor project ID")
        config_parser.add_argument(
            "--personal-note", help="add notes to private notes", action="store_true"
        )
        config_parser.add_argument(
            "-f",
            "--force-unlock",
            help="force unlock notes",
            action="store_true",
        )

    def _configure_global_arguments(self):
        """Enables the parameters
        - debug
        - verbose
        - insecure
        - no-timestamp
        """
        self._parser.add_argument(
            "-v",
            "--verbose",
            help="increase output verbosity (> INFO)",
            action="store_true",
        )
        self._parser.add_argument(
            "--debug", help="sets logging to DEBUG", action="store_true"
        )
        self._parser.add_argument("-n", "--notename")
        self._parser.add_argument(
            "-nt",
            "--no-timestamp",
            help="do not prepent timestamp to note",
            action="store_true",
        )

        self._parser.add_argument("-file", "--file", help="Local file to read")
        return self._parser

    def _parse_main_arguments_with_subparser(self):
        # Parse main parser arguments also if provided in subparser
        previous_unknown = None
        args, unknown = self._parser.parse_known_args()
        while len(unknown) and unknown != previous_unknown:
            args, unknown = self._parser.parse_known_args(unknown, args)
            previous_unknown = unknown

        if args.verbose:
            reptor_logger.logger.setLevel(logging.INFO)
            root_logger.setLevel(logging.INFO)

        if args.debug:
            reptor_logger.logger.setLevel(logging.DEBUG)
            root_logger.setLevel(logging.DEBUG)

        # Override conf from config file by CLI
        args_dict = vars(args)
        config = Config()
        for k in ["server", "project_id", "token", "insecure"]:
            config.set(k, args_dict.get(k) or config.get(k, ""))
        # Add cli options to config/cli
        config.set("cli", args_dict)

        self.logger.debug(f"Parsed args: {args}")
        return args

    def _print_title(self):
        """Prints a parsed & converted markdown text from title.md in the root directory"""
        with open(settings.BASE_DIR / "title.md", "r", encoding="utf-8") as f:
            reptor_console.print(convert_markdown_to_console(f.read()))

    def run(self) -> None:
        """The run method actually starts the cli application"""
        # Todo: Refactor the order when the parsers are available. Otherwise
        # with the current direction we don't have --debug and -verbose output
        # until we hit self._parse_main_arguments_with_subparser()
        self.logger.info("Reptor is starting...")

        # Todo: remove current hack for debug and verbose logging
        if "-v" in sys.argv or "--verbose" in sys.argv or "-verbose" in sys.argv:
            reptor_logger.logger.setLevel(logging.INFO)
            root_logger.setLevel(logging.INFO)
        if "--debug" in sys.argv or "-debug" in sys.argv:
            reptor_logger.logger.setLevel(logging.DEBUG)
            root_logger.setLevel(logging.DEBUG)

        self.plugin_manager.run_loading_sequence()

        self.plugin_manager.load_plugins()

        django_settings.configure(settings, DEBUG=True)
        django.setup()
        self._create_parsers()

        self._dynamically_add_plugin_options()

        # Static plugin options
        self._add_config_parse_options()

        self._configure_global_arguments()
        args = self._parse_main_arguments_with_subparser()

        # Configure the API
        self._api = APIManager(reptor=self)

        # Subcommands
        if args.command in self.plugin_manager.LOADED_PLUGINS:
            if self._config._no_config_file:
                self.logger.warning(
                    "No config file found. You can create one with 'reptor conf'"
                )
            try:
                plugin = self.plugin_manager.LOADED_PLUGINS[args.command]
                self.logger.debug(f"Loading Plugin: {plugin.__name__}")
                plugin.loader(reptor=self, **self._config.get("cli")).run()
            except Exception as e:
                self.logger.debug(traceback.format_exc())
                self.logger.fail(e)
        else:
            # This is called when the user uses python -m reptor or any other way
            # but provides no arguments at all
            print(self._parser.format_help())



import re

from django.template import base as django_base
from django.template.base import Token, TokenType

# HMTL comment tags
HTML_REGEX = re.compile(r"(<!--{%.*?%}-->|<!--{{.*?}}-->|<!--{#.*?#}-->)")
BLOCK_TAG_START = "<!--{%"
BLOCK_TAG_END = "%}-->"
VARIABLE_TAG_START = "<!--{{"
VARIABLE_TAG_END = "}}-->"
COMMENT_TAG_START = "<!--{#"
COMMENT_TAG_END = "#}-->"


def setup_django_tags():
    def create_token(self, token_string, position, lineno, in_tag):
        """
        Convert the given token string into a new Token object and return it.
        If in_tag is True, we are processing something that matched a tag,
        otherwise it should be treated as a literal string.
        """
        if in_tag:
            # The [0:2] and [2:-2] ranges below strip off *_TAG_START and
            # *_TAG_END. The 2's are hard-coded for performance. Using
            # len(BLOCK_TAG_START) would permit BLOCK_TAG_START to be
            # different, but it's not likely that the TAG_START values will
            # change anytime soon.
            token_start = token_string[0 : len(django_base.BLOCK_TAG_START)]
            if token_start == django_base.BLOCK_TAG_START:
                content = token_string[
                    len(django_base.BLOCK_TAG_START) : -len(django_base.BLOCK_TAG_END)
                ].strip()
                if self.verbatim:
                    # Then a verbatim block is being processed.
                    if content != self.verbatim:
                        return Token(TokenType.TEXT, token_string, position, lineno)
                    # Otherwise, the current verbatim block is ending.
                    self.verbatim = False
                elif content[:9] in ("verbatim", "verbatim "):
                    # Then a verbatim block is starting.
                    self.verbatim = "end%s" % content
                return Token(TokenType.BLOCK, content, position, lineno)
            if not self.verbatim:
                content = token_string[
                    len(django_base.BLOCK_TAG_START) : -len(django_base.BLOCK_TAG_END)
                ].strip()
                if token_start == django_base.VARIABLE_TAG_START:
                    return Token(TokenType.VAR, content, position, lineno)
                # BLOCK_TAG_START was handled above.
                assert token_start == django_base.COMMENT_TAG_START
                return Token(TokenType.COMMENT, content, position, lineno)
        return Token(TokenType.TEXT, token_string, position, lineno)

    # Monkey patch
    django_base.tag_re = HTML_REGEX
    django_base.Lexer.create_token = create_token
    django_base.BLOCK_TAG_START = BLOCK_TAG_START
    django_base.BLOCK_TAG_END = BLOCK_TAG_END
    django_base.VARIABLE_TAG_START = VARIABLE_TAG_START
    django_base.VARIABLE_TAG_END = VARIABLE_TAG_END
    django_base.COMMENT_TAG_START = COMMENT_TAG_START
    django_base.COMMENT_TAG_END = COMMENT_TAG_END



from reptor.lib.plugins.ToolBase import ToolBase


class Zap(ToolBase):
    """
    Parses ZAP generated reports in either XML or JSON

    Make sure you verify what type of findings you export.

    Recommended settings are to only export Medium and higher findings
    """

    meta = {
        "author": "Richard Schwabe",
        "name": "Zap",
        "version": "1.0",
        "license": "MIT",
        "tags": ["web", "zap"],
        "summary": "Parses ZAP reports (JSON, XML)",
    }

    def __init__(self, **kwargs):
        super().__init__(**kwargs)
        self.notename = kwargs.get("notename", "ZAP")
        self.multi_notes = True
        self.note_icon = "🌩️"
        if self.input_format == "raw":
            self.input_format = "json"

    def _parse_alert_data(self, data):
        return {
            "pluginid": data.find("pluginid").text,
            "alertRef": data.find("alertRef").text,
            "name": data.find("name").text,
            "riskcode": data.find("riskcode").text,
            "confidence": data.find("confidence").text,
            "riskdesc": data.find("riskdesc").text,
            "confidencedesc": data.find("confidencedesc").text,
            "desc": data.find("desc").text,
            "count": data.find("count").text,
            "solution": data.find("solution").text,
            "reference": data.find("reference").text,
            "cweid": data.find("cweid").text,
            "wascid": data.find("wascid").text,
            "sourceid": data.find("sourceid").text,
            "instances": [],
        }

    def _parse_instance_data(self, data):
        result = {
            "uri": data.find("uri").text,
            "method": data.find("method").text,
            "param": data.find("param").text,
            "attack": data.find("attack").text,
            "evidence": data.find("evidence").text,
            "otherinfo": data.find("otherinfo").text,
        }
        if data.find("requestheader"):
            result["requestheader"] = (data.find("requestheader").text,)
        if data.find("requestbody"):
            result["requestbody"] = (data.find("requestbody").text,)
        if data.find("responseheader"):
            result["responseheader"] = (data.find("responseheader").text,)
        return result

    def parse_xml(self):
        super().parse_xml(as_dict=False)
        return_data = list()
        for scan in self.xml_root:
            site = scan.attrib
            site["alerts"] = []

            for alert_item in scan[0]:
                alert = self._parse_alert_data(alert_item)

                for instance_item in alert_item.findall("./instances/instance"):
                    instance = self._parse_instance_data(instance_item)
                    alert["instances"].append(instance)

                site["alerts"].append(alert)

            return_data.append(site)

        self.parsed_input = return_data

    def parse_json(self):
        super().parse_json()
        parsed_input = list()
        for site in self.parsed_input.get("site", []):
            for attr in list(site.keys()):
                site[attr.strip("@")] = site.pop(attr)
            parsed_input.append(site)
        self.parsed_input = parsed_input

    def preprocess_for_template(self):
        data = dict()
        for site in self.parsed_input:
            title = f"{site['name']} ({len(site['alerts'])})"
            data[title] = {"data": [site]}
        data = dict(
            sorted(
                data.items(), key=lambda x: len(x[1]["data"][0]["alerts"]), reverse=True
            )
        )
        return data


loader = Zap



import json
import os
from pathlib import Path
from unittest.mock import MagicMock, Mock

import pytest

from reptor.lib.plugins.TestCaseToolPlugin import TestCaseToolPlugin
from reptor.models.Finding import FindingRaw
from reptor.models.ProjectDesign import ProjectDesign
from reptor.models.Project import Project

from ..Sslyze import Sslyze


class TestSslyze(TestCaseToolPlugin):
    templates_path = os.path.normpath(Path(os.path.dirname(__file__)) / "../templates")

    @pytest.fixture(autouse=True)
    def setUp(self) -> None:
        Sslyze.setup_class(
            Path(os.path.dirname(self.templates_path)), skip_user_plugins=True
        )
        self.sslyze = Sslyze(reptor=self.reptor)

    def _load_json_data(self):
        self.sslyze.input_format = "json"
        filepath = os.path.join(os.path.dirname(__file__), "./data/sslyze.json")
        with open(filepath, "r") as f:
            self.sslyze.raw_input = f.read()

    def test_generate_and_push_findings(self):
        # Patch API
        self.reptor.api.templates.search = Mock(return_value=[])

        self._load_json_data()
        # Assert "create_finding" is called if no findings exist
        self.sslyze.reptor.api.projects.get_findings = Mock(return_value=[])
        self.reptor.api.projects.create_finding = MagicMock()

        self.sslyze.generate_and_push_findings()
        assert self.reptor.api.projects.create_finding.called

        # Assert "create_finding" is not called if finding with same title exists
        finding_raw = FindingRaw({"data": {"title": "Weak SSL ciphers"}})
        self.reptor.api.projects.project = Project({"project_type": ""})
        self.reptor.api.project_designs.project_design = ProjectDesign()
        self.reptor.api.projects.get_findings = Mock(return_value=[finding_raw])
        self.reptor.api.projects.create_finding = MagicMock()

        self.sslyze.generate_and_push_findings()
        assert not self.sslyze.reptor.api.projects.create_finding.called

    def test_generate_findings(self):
        # Patch API
        self.reptor.api.templates.search = Mock(return_value=[])

        self._load_json_data()
        self.sslyze.generate_findings()
        assert len(self.sslyze.findings) == 1
        cipher = self.sslyze.findings[0]
        assert cipher.data.title.value == "Weak SSL ciphers"
        assert (
            cipher.data.description.value
            == 'We detected weak SSL/TLS ciphers on your server.\n**www.example.com:443 (127.0.0.1)**\n&nbsp;\n * <span style="color: red">DHE-RSA-AES256-SHA256</span> (TLS 1.2)\n * <span style="color: red">AES256-SHA256</span> (TLS 1.2)\n * <span style="color: red">ECDHE-RSA-AES256-SHA384</span> (TLS 1.2)\n * <span style="color: red">DHE-RSA-AES256-SHA</span> (TLS 1.2)\n * <span style="color: red">AES256-SHA</span> (TLS 1.2)\n * <span style="color: red">AES256-GCM-SHA384</span> (TLS 1.2)\n * <span style="color: red">DHE-RSA-AES128-SHA256</span> (TLS 1.2)\n * <span style="color: red">DHE-RSA-AES128-SHA</span> (TLS 1.2)\n * <span style="color: red">AES128-SHA</span> (TLS 1.2)\n * <span style="color: red">AES128-SHA256</span> (TLS 1.2)\n * <span style="color: red">AES128-GCM-SHA256</span> (TLS 1.2)\n * <span style="color: red">ECDHE-RSA-AES128-SHA256</span> (TLS 1.2)\n'
        )
        assert (
            cipher.data.affected_components.value[0].value
            == "www.example.com:443 (127.0.0.1)"
        )
        assert [r.value for r in cipher.data.references.value] == [
            "https://ssl-config.mozilla.org/",
            "https://ciphersuite.info/",
        ]

    def test_parse(self):
        result_dict = {"a": "b"}
        self.sslyze.raw_input = json.dumps(result_dict)
        self.sslyze.parse()
        assert self.sslyze.parsed_input == result_dict

    def test_preprocess_for_template(self):
        result = {
            "data": [
                {
                    "hostname": "www.example.com",
                    "port": "443",
                    "ip_address": "127.0.0.1",
                    "protocols": {
                        "tlsv1_2": {
                            "weak_ciphers": [
                                "DHE-RSA-AES256-SHA256",
                                "AES256-SHA256",
                                "ECDHE-RSA-AES256-SHA384",
                                "DHE-RSA-AES256-SHA",
                                "AES256-SHA",
                                "AES256-GCM-SHA384",
                                "DHE-RSA-AES128-SHA256",
                                "DHE-RSA-AES128-SHA",
                                "AES128-SHA",
                                "AES128-SHA256",
                                "AES128-GCM-SHA256",
                                "ECDHE-RSA-AES128-SHA256",
                            ]
                        }
                    },
                    "has_weak_ciphers": True,
                    "certinfo": {
                        "certificate_matches_hostname": True,
                        "has_sha1_in_certificate_chain": False,
                        "certificate_untrusted": [],
                    },
                    "vulnerabilities": {
                        "heartbleed": False,
                        "openssl_ccs": False,
                        "robot": False,
                    },
                    "has_vulnerabilities": False,
                    "misconfigurations": {
                        "compression": False,
                        "downgrade": False,
                        "client_renegotiation": False,
                        "no_secure_renegotiation": False,
                    },
                }
            ]
        }
        self._load_json_data()
        self.sslyze.parsed_input = None
        self.sslyze.parse()
        data = self.sslyze.preprocess_for_template()
        assert data == result

    def test_format(self):
        # result = """| Host | Port | Service | Version |"""
        self._load_json_data()

        # Protocols template
        protocols_result = '    * <span style="color: green">TLS 1.2</span>\n'
        self.sslyze.template = "protocols"
        self.sslyze.format()
        assert self.sslyze.formatted_input == protocols_result

        # certinfo template
        certinfo_result = """ * <span style="color: green">Certificate is trusted</span>
 * Certificate matches hostname: <span style="color: green">Yes</span>
 * SHA1 in certificate chain: <span style="color: green">No</span>
"""
        self.sslyze.template = "certinfo"
        self.sslyze.format()
        assert self.sslyze.formatted_input == certinfo_result

        # vulnerabilities template
        vulnerabilities_result = """ * Heartbleed: <span style="color: green">No</span>
 * Robot Attack: <span style="color: green">No</span>
 * OpenSSL CCS (CVE-2014-0224): <span style="color: green">No</span>
"""
        self.sslyze.template = "vulnerabilities"
        self.sslyze.format()
        assert self.sslyze.formatted_input == vulnerabilities_result

        # misconfigurations template
        misconfigurations_result = """ * Compression: <span style="color: green">No</span>
 * Downgrade Attack (no SCSV fallback): <span style="color: green">No</span>
 * No Secure Renegotiation: <span style="color: green">No</span>
 * Client Renegotiation: <span style="color: green">No</span>
"""
        self.sslyze.template = "misconfigurations"
        self.sslyze.format()
        assert self.sslyze.formatted_input == misconfigurations_result

        # weak ciphers template
        weak_ciphers_result = """ * <span style="color: red">DHE-RSA-AES256-SHA256</span> (TLS 1.2)
 * <span style="color: red">AES256-SHA256</span> (TLS 1.2)
 * <span style="color: red">ECDHE-RSA-AES256-SHA384</span> (TLS 1.2)
 * <span style="color: red">DHE-RSA-AES256-SHA</span> (TLS 1.2)
 * <span style="color: red">AES256-SHA</span> (TLS 1.2)
 * <span style="color: red">AES256-GCM-SHA384</span> (TLS 1.2)
 * <span style="color: red">DHE-RSA-AES128-SHA256</span> (TLS 1.2)
 * <span style="color: red">DHE-RSA-AES128-SHA</span> (TLS 1.2)
 * <span style="color: red">AES128-SHA</span> (TLS 1.2)
 * <span style="color: red">AES128-SHA256</span> (TLS 1.2)
 * <span style="color: red">AES128-GCM-SHA256</span> (TLS 1.2)
 * <span style="color: red">ECDHE-RSA-AES128-SHA256</span> (TLS 1.2)
"""
        self.sslyze.template = "weak_ciphers"
        self.sslyze.format()
        assert self.sslyze.formatted_input == weak_ciphers_result

        # summary template
        self.sslyze.template = "default_summary"
        self.sslyze.format()
        for content in [
            certinfo_result,
            vulnerabilities_result,
            misconfigurations_result,
            weak_ciphers_result,
            protocols_result,
        ]:
            assert content in self.sslyze.formatted_input

    def test_path_methods(self):
        plugin_path = Path(os.path.normpath(Path(os.path.dirname(__file__)) / ".."))
        template_paths = self.sslyze.get_plugin_dir_paths(
            plugin_path, "templates", skip_user_plugins=True
        )
        assert len(template_paths) == 1
        assert isinstance(template_paths[0], Path)
        assert template_paths[0] == plugin_path / "templates"

        templates = self.sslyze.get_filenames_from_paths(template_paths, "md")
        template_names = [
            "certinfo",
            "default_summary",
            "misconfigurations",
            "protocols",
            "vulnerabilities",
            "weak_ciphers",
        ]
        assert all([t in templates for t in template_names])



import pytest

from reptor.lib.reptor import Reptor
from reptor.models.FindingTemplate import FindingTemplate

from .. import BaseImporter


class TestBaseImporter:
    @pytest.fixture(autouse=True)
    def setUp(self):
        self.reptor = Reptor()

    def test_run(self):
        def next_findings_batch(*args, **kwargs):
            return []

        with pytest.raises(AttributeError):
            BaseImporter.BaseImporter(reptor=self.reptor).run()

        BaseImporter.BaseImporter.mapping = {"title": "title"}
        base_importer = BaseImporter.BaseImporter(reptor=self.reptor)
        base_importer.next_findings_batch = next_findings_batch
        base_importer.run()

    def test_next_findings_batch(self):
        with pytest.raises(NotImplementedError):
            BaseImporter.BaseImporter(reptor=self.reptor).next_findings_batch()

    def test_create_and_upload(self):
        class Api:
            class Templates:
                def upload_new_template(self, template, **kwargs):
                    assert template == finding
                    return FindingTemplate(finding_example)

            templates = Templates()

        def convert_old_tools_references(value):
            return value.splitlines()

        finding_example = {
            "id": "e6961177-0582-4dd2-b057-c48490294ddd",
            "created": "2022-10-19T10:30:48.055519Z",
            "updated": "2023-05-17T07:15:51.700130Z",
            "details": "http://localhost:8000/api/v1/findingtemplates/e6961177-0582-4dd2-b057-c48490294ddd",
            "lock_info": None,
            "usage_count": 1,
            "source": "imported",
            "tags": ["web"],
            "language": "en-US",
            "status": "in-progress",
            "data": {
                "title": "SQL Injection (SQLi)",
            },
        }

        base_importer = BaseImporter.BaseImporter(reptor=self.reptor)
        base_importer.convert_old_tools_references = convert_old_tools_references
        base_importer.reptor._api = Api()
        base_importer.mapping = {
            "old_tools_title": "title",
            "old_tools_description": "description",
            "old_tools_cvss_vector": "cvss",
            "old_tools_custom_field": "custom_field",
            "old_tools_empty_field": "empty_field",
            "old_tools_missing_field": "missing_field",
            "old_tools_references": "references",
            "recommendation_part_1": "recommendation",
            "recommendation_part_2": "recommendation",
            "recommendation_part_3": "recommendation",
        }

        finding_data = {
            "language": "en-US",
            "status": "in-progress",
            "data": {
                "old_tools_title": "my title",
                "old_tools_description": "my description",
                "old_tools_cvss_vector": "CVSS:3.0/AV:N/AC:L/PR:L/UI:R/S:U/C:L/I:H/A:H",
                "old_tools_custom_field": "custom info",
                "old_tools_empty_field": "",
                "old_tools_references": "reference1\nreference2\r\nreference3",
                "recommendation_part_1": "do this!",
                "recommendation_part_2": "do that!",
                "recommendation_part_3": "do nothing!",
            },
        }
        finding = base_importer._create_finding_item(finding_data)
        assert isinstance(finding, FindingTemplate)
        assert finding.translations[0].language == "en-US"
        assert finding.translations[0].status == "in-progress"

        assert finding.translations[0].data.title == "my title"
        assert finding.translations[0].data.description == "my description"
        assert (
            finding.translations[0].data.cvss
            == "CVSS:3.0/AV:N/AC:L/PR:L/UI:R/S:U/C:L/I:H/A:H"
        )
        assert finding.translations[0].data.custom_field == "custom info"
        assert finding.translations[0].data.empty_field == ""
        with pytest.raises(AttributeError):
            finding.translations[0].data.missing_field
        assert finding.translations[0].data.references == [
            "reference1",
            "reference2",
            "reference3",
        ]
        assert (
            finding.translations[0].data.recommendation
            == "do this!\ndo that!\ndo nothing!"
        )

        base_importer._upload_finding_template(finding)



from reptor.models.Base import BaseModel


import typing


class User(BaseModel):
    """
    Attributes:
        username:
        name:
        title_before:
        first_name:
        middle_name:
        last_name:
        title_after:
        is_active:
        roles:
        email:
        phone:
        mobile:
        scope:
        is_superuser:
        is_designer:
        is_template_editor:
        is_guest:
        is_user_manager:
        is_system_user:
        is_global_archiver:
        is_mfa_enabled:
        can_login_local:
        can_login_sso:
    """

    username: str = ""
    name: str = ""
    title_before: str = ""
    first_name: str = ""
    middle_name: str = ""
    last_name: str = ""
    title_after: str = ""
    is_active: bool = False
    roles: typing.List[str] = []
    email: str = ""
    phone: str = ""
    mobile: str = ""
    scope: typing.List[str] = []
    is_superuser: bool = False
    is_designer: bool = False
    is_template_editor: bool = False
    is_guest: bool = False
    is_user_manager: bool = False
    is_system_user: bool = False
    is_global_archiver: bool = False
    is_mfa_enabled: bool = False
    can_login_local: bool = False
    can_login_sso: bool = False



import glob
import json
import logging
import os
import sys
import typing
from pathlib import Path
from xml.etree import ElementTree

import toml
import xmltodict
from django.template import Context, Template
from django.template.loader import render_to_string

import reptor.settings as settings
from reptor.lib.exceptions import IncompatibleDesignException
from reptor.models.Finding import Finding

from .Base import Base

log = logging.getLogger("reptor")


class ToolBase(Base):
    """The ToolBase provides plugin developers with all functionality that
    is commonly encountered when writing a plugin.


    Attributes:
        raw_input: Unformatted input, either from stdin or a file
        template: The .md file to be used during formatting
    """

    FINDING_PREFIX = "finding_"

    def __init__(self, **kwargs):
        super().__init__(**kwargs)
        self.action = kwargs.get("action")
        self.push_findings = kwargs.get("push_findings")
        self.note_icon = "🛠️"
        self.multi_notes = kwargs.get("multi_notes", False)
        self.raw_input = None
        self.parsed_input = None
        self.formatted_input = None
        self.findings = []
        self.no_timestamp = (
            self.reptor.get_config().get("cli", dict()).get("no_timestamp")
        )
        self.force_unlock = (
            self.reptor.get_config().get("cli", dict()).get("force_unlock")
        )

        self.input_format = kwargs.get("format")
        self.template = kwargs.get("template", self.template)

        if self.template_paths:
            settings.TEMPLATES[0]["DIRS"] = self.template_paths

    @classmethod
    def get_plugin_dir_paths(
        cls,
        plugin_path: Path,
        dirname: str,
        skip_user_plugins: bool = False,
    ) -> typing.List[Path]:
        dir_paths = list()
        if skip_user_plugins:
            user_plugin_path = None
        else:
            user_plugin_path = settings.PLUGIN_DIRS_USER / os.path.basename(plugin_path)
        for path in [
            user_plugin_path,
            plugin_path,
        ]:  # Keep order: files from user directory override
            if path is None:
                continue
            dir_path = os.path.normpath(Path(path) / dirname)

            if dir_path not in dir_paths and os.path.isdir(dir_path):
                dir_paths.append(Path(dir_path))

        # Return list of existing template paths
        return dir_paths

    @classmethod
    def get_filenames_from_paths(cls, dir_paths: list, filetype: str):
        """
        takes a list of paths and returns a list of filenames without file extension
        """
        filetype = f"*.{filetype.strip('.')}"
        # Get template names from paths
        files_from_plugin_dir = list()
        for path in dir_paths:
            files = [
                os.path.basename(f).rsplit(".", 1)[0]
                for f in glob.glob(os.path.join(path, filetype))
            ]
            files_from_plugin_dir.extend(
                [t for t in files if t not in files_from_plugin_dir]
            )
        return files_from_plugin_dir

    @classmethod
    def setup_class(cls, plugin_path: Path, skip_user_plugins: bool = False):
        # Get template and finding paths from plugin and userdir
        cls.finding_paths = cls.get_plugin_dir_paths(
            plugin_path,
            settings.FINDING_TEMPLATES_DIR_NAME,
            skip_user_plugins=skip_user_plugins,
        )
        cls.template_paths = cls.get_plugin_dir_paths(
            plugin_path,
            settings.PLUGIN_TEMPLATES_DIR_NAME,
            skip_user_plugins=skip_user_plugins,
        )
        cls.templates = cls.get_filenames_from_paths(cls.template_paths, "md")

        # Choose default template
        cls.template = None
        if cls.templates:
            if len(cls.templates) == 1:
                cls.template = cls.templates[0]
            elif len(cls.templates) > 1:
                default_templates = [t for t in cls.templates if "default" in t]
                try:
                    cls.template = default_templates[0]
                except IndexError:
                    cls.template = cls.templates[0]

    @classmethod
    def add_arguments(cls, parser, plugin_filepath=None):
        super().add_arguments(parser, plugin_filepath)
        if plugin_filepath:
            cls.setup_class(Path(os.path.dirname(plugin_filepath)))

        if cls.templates:
            parser.add_argument(
                "-t",
                "--template",
                action="store",
                default=cls.template,
                choices=cls.templates,
                help="Template for output formatting",
            )

        action_group = parser.add_mutually_exclusive_group()
        action_group.title = "action_group"
        action_group.add_argument(
            "-parse",
            "--parse",
            action="store_const",
            dest="action",
            const="parse",
            default="format",
        )
        if cls.templates:
            action_group.add_argument(
                "-format",
                "--format",
                action="store_const",
                dest="action",
                const="format",
                default="format",
            )
            action_group.add_argument(
                "-upload",
                "--upload",
                action="store_const",
                dest="action",
                const="upload",
                default="format",
            )

        if cls._get_finding_methods():
            action_group.add_argument(
                "-push-findings",
                "--push-findings",
                action="store_true",
            )

        if any(
            [
                cls.parse_xml != ToolBase.parse_xml,
                cls.parse_json != ToolBase.parse_json,
                cls.parse_csv != ToolBase.parse_csv,
            ]
        ):  # Prevent adding input_format_group if no parsing methods are implemented
            input_format_group = parser.add_mutually_exclusive_group()
            input_format_group.title = "input_format_group"

            # Add parsing options only if implemented by modules
            if cls.parse_xml != ToolBase.parse_xml:
                input_format_group.add_argument(
                    "-xml",
                    "--xml",
                    action="store_const",
                    dest="format",
                    const="xml",
                    default="raw",
                )
            if cls.parse_json != ToolBase.parse_json:
                input_format_group.add_argument(
                    "-json",
                    "--json",
                    action="store_const",
                    dest="format",
                    const="json",
                    default="raw",
                )
            if cls.parse_csv != ToolBase.parse_csv:
                input_format_group.add_argument(
                    "-csv",
                    "--csv",
                    action="store_const",
                    dest="format",
                    const="csv",
                    default="raw",
                )

    @classmethod
    def get_input_format_group(cls, parser):
        # Find input_format_group
        for group in parser._mutually_exclusive_groups:
            if group.title == "input_format_group":
                return group

    @classmethod
    def _get_finding_methods(cls) -> typing.List[typing.Callable]:
        finding_method_names = [
            func
            for func in dir(cls)
            if callable(getattr(cls, func)) and func.startswith(cls.FINDING_PREFIX)
        ]
        return [getattr(cls, n) for n in finding_method_names]

    def run(self):
        """
        The run method is always called by the main reptor application.

        The flow is:

        Parsing -> Formatting -> Uploading
        """
        if self.push_findings:
            self.generate_and_push_findings()
            if self.action == "upload":
                # Also upload to notes
                self.upload()
            return

        if self.action == "parse":
            self.parse()
            self.console.print(self.parsed_input)
        elif self.action == "format":
            self.format()
            if self.multi_notes:
                for title, content in self.formatted_input.items():
                    self.console.print(f"### {title}\n\n{content}")
            else:
                self.console.print(self.formatted_input)
        elif self.action == "upload":
            self.upload()

    def load(self):
        """Puts the stdin into raw_input"""
        self.raw_input = sys.stdin.read()

    def parse_xml(self, as_dict=True):
        if as_dict:
            self.parsed_input = xmltodict.parse(self.raw_input)
        else:
            if not self.file_path and self.raw_input:
                self.xml_root = ElementTree.fromstring(self.raw_input)
            else:
                self.xml_root = ElementTree.parse(self.file_path).getroot()

    def parse_json(self):
        self.parsed_input = json.loads(self.raw_input)

    def parse_csv(self):
        raise NotImplementedError("Parse csv data is not implemented for this plugin.")

    def parse_raw(self):
        raise NotImplementedError("Parse raw data is not implemented for this plugin.")

    def parse(self):
        """
        Directs the input to the correct sub parsing method. For every toolbase
        plugin it is possible to handle --xml, --csv, --json or raw input.

        Depending on the arguments the corresponding sub parser method is called.

        If you decide not to support one of these, it won't be possible to provide
        the corresponding argument.
        """
        if not self.raw_input and not self.file_path:
            self.load()

        if self.input_format == "xml":
            self.parse_xml()
        elif self.input_format == "json":
            self.parse_json()
        elif self.input_format == "csv":
            self.parse_csv()

    def format(self):
        """Checks if `self.parsed_input` is set.
        If not it starts the parsing process.

        Once there is parsed data it is run through the Django Templating Engine.

        The template is set via `self.template`.

        """
        if not self.parsed_input:
            self.parse()

        data = self.preprocess_for_template()
        if self.multi_notes:
            self.formatted_input = dict()
            for title, data in data.items():
                self.formatted_input[title] = render_to_string(
                    f"{self.template}.md", data
                )
        else:
            self.formatted_input = render_to_string(f"{self.template}.md", data)

    def preprocess_for_template(self) -> typing.Optional[dict]:
        return {"data": self.parsed_input}

    def upload(self):
        """Uploads the `self.formatted_input` to sysreptor via the NotesAPI."""
        if not self.formatted_input:
            self.format()
        notename = self.notename or self.__class__.__name__.lower()

        if self.multi_notes:
            for title, content in self.formatted_input.items():
                self.reptor.api.notes.write_note(
                    content=content,
                    notename=title,
                    icon=self.note_icon,
                    parent_notename=notename,
                    no_timestamp=self.no_timestamp,
                    force_unlock=self.force_unlock,
                )
        else:
            parent_notename = "Uploads" if notename != "Uploads" else ""
            self.reptor.api.notes.write_note(
                content=self.formatted_input,
                notename=notename,
                parent_notename=parent_notename,
                icon=self.note_icon,
                no_timestamp=self.no_timestamp,
                force_unlock=self.force_unlock,
            )
        self.log.success("Successfully uploaded to notes.")

    def generate_and_push_findings(self) -> None:
        self.generate_findings()
        if len(self.findings) == 0:
            self.log.display("No findings generated.")
            return
        project_findings = [
            Finding(f, project_design=self.reptor.api.project_designs.project_design)
            for f in self.reptor.api.projects.get_findings()
        ]
        project_finding_titles = [f.data.title.value for f in project_findings]
        project_findings_from_templates = [f.template for f in project_findings]
        for finding in self.findings:
            self.log.info(f'Checking if finding "{finding.data.title.value}" exists')
            if finding.template:
                if finding.template in project_findings_from_templates:
                    self.log.display(
                        f'Finding "{finding.data.title.value}" already created from template. Skipping.'
                    )
                    continue
            elif finding.data.title.value in project_finding_titles:
                self.log.display(
                    f'Finding "{finding.data.title.value}" already exists. Skipping.'
                )
                continue

            self.log.info(f'Pushing finding "{finding.data.title.value}"')
            if finding.template:
                # First create from template to keep template reference
                created_finding = self.reptor.api.projects.create_finding_from_template(
                    finding.template
                )
                # ...then update and add data
                self.reptor.api.projects.update_finding(
                    created_finding.id, finding.to_dict()
                )
            else:
                self.reptor.api.projects.create_finding(finding.to_dict())
            self.log.success(f'Pushed finding "{finding.data.title.value}"')

    def generate_findings(self) -> typing.List[Finding]:
        """Generates findings from the parsed input.

        The findings are generated from the `self.parsed_input` and are
        written to the `self.findings` list
        """
        if not self.parsed_input:
            self.parse()
        project_design = None
        self.findings = list()
        finding_methods = self._get_finding_methods()
        for method in finding_methods:
            finding = None
            finding_context = method(self)
            if finding_context is None:
                # Don't create finding if method returns None
                continue

            finding_name = method.__name__[len(self.FINDING_PREFIX) :]

            # Check if remote finding exists
            template_tag = f"{self.__class__.__name__.lower()}:{finding_name}"
            for finding_template in self.reptor.api.templates.search(template_tag):
                if template_tag in finding_template.tags:
                    # Take first matching template and fetch full data (search returns partial data only)
                    self.log.info(f"Found remote finding template for {template_tag}.")
                    finding_template = self.reptor.api.templates.get_template(
                        finding_template.id
                    )
                    # Get project language
                    language = self.reptor.api.projects.project.language
                    try:
                        translation = [
                            t
                            for t in finding_template.translations
                            if t.language == language
                        ][0]
                    except IndexError:
                        translation = [
                            t for t in finding_template.translations if t.is_main
                        ][0]
                        self.log.display(
                            f"No translation found for {language}. Taking main translation {translation.language}."
                        )

                    finding = Finding.from_translation(
                        translation, force_compatible=False
                    )
                    finding.template = finding_template.id
                    break

            if not finding:
                # Check if findings toml exists
                template_dict = self.get_local_template_data(finding_name)
                if not template_dict:
                    self.log.warning(
                        f"Did not find finding template for {finding_name}. Creating default finding."
                    )
                    description = (
                        f"```{json.dumps(finding_context, indent=2)}```"
                        if finding_context
                        else "No description"
                    )
                    template_dict = {
                        "data": {
                            "title": finding_name.replace("_", " ").title(),
                            "description": description,
                        },
                    }

                try:
                    finding = Finding(
                        template_dict,
                        project_design=project_design,
                    )
                except IncompatibleDesignException:
                    self.log.info(
                        "Finding data not compatible with project design. Fetching project design from project."
                    )
                    project_design = self.reptor.api.project_designs.project_design
                    finding = Finding(
                        template_dict,
                        project_design=project_design,
                        force_compatible=False,
                    )

            django_context = Context(finding_context)
            for finding_data in finding.data:
                # Iterate over all finding fields
                if isinstance(finding_data.value, list):
                    # Iterate over list to render list items
                    if finding_data.name == "affected_components":
                        finding_data.value = finding_context.get(
                            "affected_components", []
                        )
                        continue
                    finding_data_list = list()
                    for v in finding_data.value:
                        if content := Template(v.value).render(django_context):
                            finding_data_list.append(content)
                    finding_data.value = finding_data_list
                elif finding_data.value:
                    # If value not empty, render template
                    finding_data.value = Template(finding_data.value).render(
                        django_context
                    )
            self.findings.append(finding)
        return self.findings

    def get_local_template_data(self, name: str) -> typing.Optional[dict]:
        """Loads a finding template from the local findings directory.

        Args:
            name: The name of the finding template to load

        Returns:
            A FindingRaw object or None if the template does not exist
        """
        if not self.finding_paths:
            return None
        if not name.endswith(".toml"):
            name = f"{name}.toml"

        for path in self.finding_paths:
            finding_template_path = path / name
            if os.path.isfile(finding_template_path):
                with open(finding_template_path, "r") as f:
                    try:
                        finding_template = toml.load(f)
                    except (toml.TomlDecodeError, TypeError) as e:
                        self.log.warning(
                            f"Error while loading toml finding template {name}."
                        )
                        return None
                return finding_template
        return None



import typing
from posixpath import join as urljoin
from uuid import UUID

import yaml

from .. import settings as settings
from .logger import reptor_logger


class Config:
    _raw_config: dict = {
        "server": "https://demo.sysre.pt",
        "token": None,
        "project_id": None,
        "log_file": False,
    }
    _no_config_file: bool = False

    """These keys are ignored when writing the config to a file
    By default this should be cli and insecure
    """
    _ignored_keys_in_config = ["cli", "insecure"]

    def __new__(cls):
        """Make the Config Singleton

        This helps to ensure, that there cannot be two different
        configurations while running the program.

        Returns:
            Config: current Config Object
        """
        if not hasattr(cls, "instance"):
            cls.instance = super(Config, cls).__new__(cls)
        return cls.instance

    def get(
        self, key: str, default: typing.Optional[typing.Any] = None, plugin: str = ""
    ) -> typing.Any:
        """Returns a value from the current config

        Args:
            key (_type_): _description_
            default (typing.Any, optional): _description_. Defaults to None.

        Returns:
            typing.Any: _description_
        """
        if plugin:
            return (self._raw_config.get(plugin) or dict()).get(key, default)
        else:
            return self._raw_config.get(key, default)

    def get_config_keys(self, plugin: str = "") -> typing.Collection:
        if plugin:
            return (self._raw_config.get(plugin) or dict()).keys()
        else:
            return self._raw_config.keys()

    def set(self, key: str, value: typing.Any, plugin: str = ""):
        """Sets a value in the config

        Args:
            key (typing.Any): _description_
            value (typing.Any): _description_
        """
        if plugin:
            plugin_settings = self._raw_config.get(plugin, {})
            plugin_settings.update({key: value})
            self._raw_config.update({plugin: plugin_settings})
        else:
            self._raw_config.update({key: value})

    def load_config(self):
        """Loads config file from user home directory"""
        if not settings.PERSONAL_SYSREPTOR_HOME.exists():
            reptor_logger.highlight(
                "No .sysreptor folder found in home directory...Creating one"
            )
            # exist_ok=True because logger might be faster, when creating log file and parents=True
            settings.PERSONAL_SYSREPTOR_HOME.mkdir(exist_ok=True)
        try:
            with open(settings.PERSONAL_CONFIG_FILE, "r") as f:
                self._raw_config = yaml.safe_load(f.read())
        except FileNotFoundError:
            self._no_config_file = True

    def get_config_from_user(self):
        """Asks the user for the individiual settings and offers to
        write them into a config file
        """
        default_server = self._raw_config.get("server")
        self._raw_config["server"] = (
            input(f"Server [{default_server}]: ") or default_server
        )

        default_api_token = (
            self._raw_config.get("token")
            or f'Create at {urljoin(self._raw_config["server"], "users/self/apitokens/")}'
            if self._raw_config["server"].startswith("http")
            else ""
        )
        self._raw_config["token"] = input(
            f"API Token{ f' [{default_api_token}]' if default_api_token else ''}: "
        ) or self._raw_config.get("token")

        default_project_id = self._raw_config.get("project_id")
        self._raw_config["project_id"] = (
            input(
                f"Project ID{ f' [{default_project_id}]' if default_project_id else ''}: "
            )
            or default_project_id
        )

        self.store_config()

    def store_config(self):
        """Offer user to write temporary config to local config file"""
        store = None
        while store not in ["y", "n"]:
            store = input(
                f"Store to config to {settings.PERSONAL_CONFIG_FILE}? [y/n]: "
            )[:1].lower()
        if store == "y":
            self._write_to_file()
        self.instance._raw_config = self._raw_config

    def _write_to_file(self):
        """Writes config file as yaml file"""
        if self._raw_config:
            settings.PERSONAL_SYSREPTOR_HOME.mkdir(exist_ok=True)
            with open(settings.PERSONAL_CONFIG_FILE, "w") as f:
                filtered_config = {
                    k: v
                    for k, v in self._raw_config.items()
                    if k not in self._ignored_keys_in_config
                }
                yaml.dump(filtered_config, f, encoding="utf-8")
        else:
            raise ValueError("No config is currently set")

    def get_server(self) -> str:
        """Always returns the server url without the final /

        When you concat the server url you are responsible for starting with /

        i.e:

            f"{self.reptor.get_config().get_server()}/api/v1/pentestprojects"

        Returns:
            str: _description_
        """
        server_url = self.get("server", "")
        if server_url[-1:] == "/":
            server_url = server_url[:-1]
        if not server_url:
            raise ValueError(
                "No SysReptor server. Try 'reptor conf' or use '--server'."
            )
        return server_url

    def get_token(self) -> str:
        """The Token is required to connect to the Rest API

        Returns:
            str: Token to Authenticate
        """
        if token := self.get("token"):
            return token
        else:
            raise ValueError(
                "No SysReptor API token. Try 'reptor conf' or use '--token'."
            )

    def get_project_id(self) -> str:
        """Do not use this, instead use self.reptor.get_active_project_id()
        This will query the Project ID from the actual config.
        Can be overwritten via CLI
        Returns:
            str: Project ID
        """
        if project_id := self.get("project_id"):
            try:
                UUID(project_id)
                return project_id
            except ValueError:
                raise ValueError(f"Project ID ('{project_id}') is not a valid UUID.")
        return ""

    def get_cli_overwrite(self) -> typing.Dict:
        """Gives access to the entire CLI arguments.
        If not found, it will return an empty dict

        Returns:
            typing.Dict: Holds all CLI arguments
        """
        return self.get("cli", {})

    def get_log_file(self) -> bool:
        """Checks if the user wants to keep a log file in their home directory

        Returns:
            bool: True to keep logs in a file
        """
        return self.get("log_file", False)



from unittest.mock import Mock

import django
from django.conf import settings
from django.template import engines

from reptor.api.APIClient import APIClient
from reptor.api.manager import APIManager
from reptor.lib.conf import Config
from reptor.lib.conf import settings as reptor_settings
from reptor.lib.reptor import Reptor

settings.configure(reptor_settings, DEBUG=True)
django.setup()


class TestCaseToolPlugin:
    @classmethod
    def setup_class(cls):
        cls.reptor = Reptor()
        cls.patch_apis()

        settings.TEMPLATES[0]["DIRS"] = [cls.templates_path]
        try:
            engines._engines["django"].engine.dirs = [cls.templates_path]
        except KeyError:
            pass

    @classmethod
    def teardown_class(cls):
        settings.TEMPLATES[0]["DIRS"] = []
        try:
            engines._engines["django"].engine.dirs = []
        except KeyError:
            pass

    @classmethod
    def patch_apis(cls):
        cls.reptor._api = APIManager(reptor=cls.reptor)
        cls.reptor._config = Config()
        # cls.reptor._config.load_config()
        cls.reptor._config._raw_config = {
            "project_id": "db837c68-ff58-4f63-9161-d2310d71999b",
            "server": "https://demo.sysre.pt",
            "token": "sysreptor_ABCDEFGHIJKLMNOPQRSTUVWXYZ1234567890",
        }
        APIClient.get = APIClient.post = APIClient.put = APIClient.patch = Mock(
            side_effect=RuntimeError(
                "APIClient cannot be used in tests. Patch methods as needed."
            )
        )



import json

from reptor.models.Finding import Finding
from reptor.models.Project import Project
from reptor.models.Section import Section
from reptor.models.User import User


class TestProjectModelParsing:
    example_project = """
    {
            "id": "4cf78324-8502-4fb0-936a-724892d3c539",
            "created": "2023-06-16T19:12:00.478314Z",
            "updated": "2023-06-25T13:49:48.935500Z",
            "name": "PNPT Exam",
            "project_type": "fa670018-e6ef-4b73-989b-1e4c4af09cee",
            "language": "en-US",
            "tags": [],
            "readonly": true,
            "source": "created",
            "copy_of": null,
            "members": [
                {
                    "id": "f2c9bad4-c916-4c18-9f76-d5ef94b34453",
                    "username": "reptor",
                    "name": "Bjoern Schwabe",
                    "title_before": "",
                    "first_name": "Bjoern",
                    "middle_name": null,
                    "last_name": "Schwabe",
                    "title_after": "",
                    "is_active": true,
                    "roles": [
                        "pentester"
                    ]
                },
                {
                    "id": "f2c9bad4-c916-4c18-9f76-d5ef94b34454",
                    "username": "richard",
                    "name": "Richard Schwabe",
                    "title_before": "",
                    "first_name": "Richard",
                    "middle_name": null,
                    "last_name": "Schwabe",
                    "title_after": "",
                    "is_active": true,
                    "roles": [
                        "pentester"
                    ]
                }
            ],
            "imported_members": [],
            "details": "http://localhost:8000/api/v1/pentestprojects/4cf78324-8502-4fb0-936a-724892d3c539",
            "findings": [{"id": "a9e6fd6b-773f-46a8-8a5c-58fdeefbdfc0", "created": "2023-08-25T11:57:03.886931Z", "updated": "2023-08-25T11:57:03.889063Z", "project": "4820bd5d-51f1-4dca-a4a4-78ba935b615c", "project_type": "50dfad6e-805d-4215-b098-a73e03b1ec3b", "language": "en-US", "lock_info": null, "template": null, "assignee": null, "status": "in-progress", "order": 1, "data": {"cvss": "CVSS:3.1/AV:N/AC:L/PR:N/UI:N/S:U/C:L/I:L/A:N", "title": "Reflected Cross-Site Scripting (XSS)", "impact": "", "summary": "We detected a reflected XSS vulnerability.\\n\\n**XSS targets**\\n\\n* https://example.com/alert(1)\\n* https://example.com/q=alert(1)\\n", "severity": null, "references": ["https://owasp.org/www-community/attacks/xss/"], "description": "", "precondition": "", "retest_notes": "", "retest_status": null, "wstg_category": null, "recommendation": "HTML encode user-supplied inputs.", "owasp_top10_2021": null, "affected_components": ["https://example.com/alert(1)", "https://example.com/q=alert(1)"], "short_recommendation": ""}}],
            "sections": [{"id": "other", "label": "Executive Summary", "fields": ["title"], "project": "4820bd5d-51f1-4dca-a4a4-78ba935b615c", "project_type": "50dfad6e-805d-4215-b098-a73e03b1ec3b", "language": "en-US", "lock_info": null, "assignee": null, "status": "in-progress", "data": {"title": "Test"}}],
            "notes": "http://localhost:8000/api/v1/pentestprojects/4cf78324-8502-4fb0-936a-724892d3c539/notes",
            "images": "http://localhost:8000/api/v1/pentestprojects/4cf78324-8502-4fb0-936a-724892d3c539/images"
        }"""

    def test_project_parsing(self):
        api_test_data = json.loads(self.example_project)

        test_project = Project(api_test_data)
        assert test_project.id == "4cf78324-8502-4fb0-936a-724892d3c539"
        assert test_project.members[0].id == "f2c9bad4-c916-4c18-9f76-d5ef94b34453"
        assert test_project.members[1].username == "richard"
        assert test_project.project_type == "fa670018-e6ef-4b73-989b-1e4c4af09cee"

        assert isinstance(test_project.sections[0], Section)
        assert isinstance(test_project.findings[0], Finding)
        assert isinstance(test_project.members[0], User)

        test_project_dict = test_project.to_dict()
        # Ensure items are still as before to_dict
        assert isinstance(test_project.sections[0], Section)
        assert isinstance(test_project.findings[0], Finding)
        assert isinstance(test_project.members[0], User)

        assert isinstance(test_project_dict["sections"], list)
        assert isinstance(test_project_dict["findings"], list)
        assert isinstance(test_project_dict["members"], list)

        assert (
            test_project.findings[0].data.title.value
            == "Reflected Cross-Site Scripting (XSS)"
        )
        assert test_project.sections[0].data.title.value == "Test"

        assert (
            test_project_dict["findings"][0]["data"]["title"]
            == "Reflected Cross-Site Scripting (XSS)"
        )
        assert test_project_dict["sections"][0]["data"]["title"] == "Test"



import json

import pytest

from reptor.models.FindingTemplate import FindingTemplate
from reptor.models.ProjectDesign import ProjectDesign, ProjectDesignField
from reptor.models.Section import SectionData, SectionDataField, SectionRaw


class TestSectionModelParsing:
    example_design_with_report_fields_only = """
    {
	"report_fields": {
		"draft": {
			"type": "boolean",
			"label": "Is Draft?",
			"origin": "custom",
			"default": true
		},
		"title": {
			"type": "string",
			"label": "Title",
			"origin": "core",
			"default": "TODO report title",
			"required": true,
			"spellcheck": false
		},
		"report_date": {
			"type": "date",
			"label": "Report Date",
			"origin": "custom",
			"default": null,
			"required": true
		},
		"report_version": {
			"type": "string",
			"label": "Report Version",
			"origin": "custom",
			"default": "1.0",
			"required": true,
			"spellcheck": false
		},
		"list_of_changes": {
			"type": "list",
			"items": {
				"type": "object",
				"label": "",
				"origin": "custom",
				"properties": {
					"date": {
						"type": "date",
						"label": "Date",
						"origin": "custom",
						"default": null,
						"required": true
					},
					"version": {
						"type": "string",
						"label": "Version",
						"origin": "custom",
						"default": "TODO version",
						"required": true,
						"spellcheck": false
					},
					"description": {
						"type": "string",
						"label": "Description",
						"origin": "custom",
						"default": "TODO description",
						"required": true,
						"spellcheck": false
					}
				}
			},
			"label": "List of Changes",
			"origin": "custom",
			"required": true   
        }
    }}
    """

    example_section = """
    {
        "id": "other",
        "label": "Other",
        "fields": [
            "title",
            "report_date",
            "report_version",
            "list_of_changes",
            "draft"
        ],
        "project": "42c2f73a-4383-4ec2-a3fa-281598edb0e8",
        "project_type": "53c185f9-b8f3-44e3-ba4d-03a960cb795a",
        "language": "en-US",
        "lock_info": null,
        "assignee": null,
        "status": "in-progress",
        "data": {
            "title": "Test",
            "report_date": "2023-07-31",
            "report_version": "1.0",
            "list_of_changes": [
                {
                    "date": "2023-07-24",
                    "version": "1.0",
                    "description": "Description"
                },
                {
                    "date": "2023-07-25",
                    "version": "2.0",
                    "description": "New Description"
                }
            ],
            "draft": true
        }
    }"""

    example_finding_template = '{"id":"82c5d4c5-b1ff-4d17-9567-c2679df325d5","created":"2022-10-19T10:30:48.055000Z","updated":"2023-07-24T10:19:59.815926Z","details":"https://syslifters.sysre.pt/api/v1/findingtemplates/82c5d4c5-b1ff-4d17-9567-c2679df325d5","images":"https://syslifters.sysre.pt/api/v1/findingtemplates/82c5d4c5-b1ff-4d17-9567-c2679df325d5/images","usage_count":14,"source":"created","tags":["web"],"translations":[{"id":"7171d2ef-a2ca-4710-9fea-6689ff2b9a26","created":"2023-07-24T10:19:59.813648Z","updated":"2023-07-24T10:19:59.813885Z","language":"en-US","status":"finished","is_main":true,"risk_score":9.8,"risk_level":"critical","data":{"title":"SQL Injection (SQLi)","cvss":"CVSS:3.1/AV:N/AC:L/PR:N/UI:N/S:U/C:H/I:H/A:H","summary":"The web application processed user input in an insecure manner and was thus vulnerable to SQL injection. In an SQL injection attack, special input values in the web application are used to influence the application\'s SQL statements to its database. Depending on the database used and the design of the application, this may make it possible to read and modify the data stored in the database, perform administrative actions (e.g., shut down the DBMS), or in some cases even gain code execution and the accompanying complete control over the vulnerable server.","description":"We identified an SQL injection vulnerability in the web application and were able to access stored data in the database as a result.\\n\\n**TODO: technical description**\\n\\nSQL Injection is a common server-side vulnerability in web applications. It occurs when software developers create dynamic database queries that contain user input. In an attack, user input is crafted in such a way that the originally intended action of an SQL statement is changed. SQL injection vulnerabilities result from an application\'s failure to dynamically create database queries insecurely and to properly validate user input. They are based on the fact that the SQL language basically does not distinguish between control characters and data characters. In order to use a control character in the data part of an SQL statement, it must be encoded or escaped appropriately beforehand.\\n\\nAn SQL injection attack is therefore essentially carried out by inserting a control character such as `\'` (single apostrophe) into the user input to place new commands that were not present in the original SQL statement. A simple example will demonstrate this process. The following SELECT statement contains a variable userId. The purpose of this statement is to get data of a user with a specific user id from the Users table.\\n\\n`sqlStmnt = \'SELECT * FROM Users WHERE UserId = \' + userId;`\\n\\nAn attacker could now use special user input to change the original intent of the SQL statement. For example, he could use the string `\' or 1=1` as user input. In this case, the application would construct the following SQL statement:\\n\\n`sqlStmnt = \'SELECT * FROM Users WHERE UserId = \' + \' or 1=1;`\\n\\nInstead of the data of a user with a specific user ID, the data of all users in the table is now returned to the attacker after executing the statement. This gives an attacker the ability to control the SQL statement in his own favor. \\n\\nThere are a number of variants of SQL injection vulnerabilities, attacks and techniques that occur in different situations and depending on the database system used. However, what they all have in common is that, as in the example above, user input is always used to dynamically construct SQL statements. Successful SQL injection attacks can have far-reaching consequences. One would be the loss of confidentiality and integrity of the stored data. Attackers could gain read and possibly write access to sensitive data in the database. SQL injection could also compromise the authentication and authorization of the web application, allowing attackers to bypass existing access controls. In some cases, SQL injection can also be used to gain code execution, allowing an attacker to gain complete control over the vulnerable server.","precondition":null,"impact":null,"recommendation":"* Use prepared statements throughout the application to effectively avoid SQL injection vulnerabilities. Prepared statements are parameterized statements and ensure that even if input values are manipulated, an attacker is unable to change the original intent of an SQL statement. \\n* Use existing stored procedures by default where possible. Typically, stored procedures are implemented as secure parameterized queries and thus protect against SQL injections.\\n* Always validate all user input. Ensure that only input that is expected and valid for the application is accepted. You should not sanitize potentially malicious input.\\n* To reduce the potential damage of a successful SQL Injection attack, you should minimize the assigned privileges of the database user used according to the principle of least privilege.\\n* For detailed information and assistance on how to prevent SQL Injection vulnerabilities, see OWASP\'s linked SQL Injection Prevention Cheat Sheet.","short_recommendation":"Make sure that Prepared Statements and Stored Procedures (where possible) are used throughout the application. This prevents the originally intended action of an SQL statement from being manipulated by an attacker.","references":["https://www.owasp.org/index.php/SQL_Injection_Prevention_Cheat_Sheet"],"affected_components":[],"owasp_top10_2021":null,"wstg_category":null,"retest_notes":null,"retest_status":null,"retest":null,"retest_state":null}},{"id":"e1aa87ee-7eed-4f47-b730-ab4b255d7abe","created":"2023-08-11T11:23:04.659627Z","updated":"2023-08-11T11:23:04.661221Z","language":"de-DE","status":"in-progress","is_main":false,"risk_score":0.0,"risk_level":"info","data":{"title":"Template Titel","summary":"Auch test."}}],"lock_info":null}'

    def test_section_data(self):
        json_example = json.loads(self.example_section)
        section_raw = SectionRaw(json_example)
        project_design = ProjectDesign(
            json.loads(self.example_design_with_report_fields_only)
        )
        section_data = SectionData(project_design.report_fields, section_raw.data)
        assert section_data.draft.name == "draft"
        assert section_data.draft.type == "boolean"
        assert section_data.draft.value == True

        assert section_data.title.name == "title"
        assert section_data.title.type == "string"
        assert section_data.title.value == "Test"

        assert section_data.report_date.name == "report_date"
        assert section_data.report_date.type == "date"
        assert section_data.report_date.value == "2023-07-31"

        assert section_data.report_version.name == "report_version"
        assert section_data.report_version.type == "string"
        assert section_data.report_version.value == "1.0"

        assert section_data.list_of_changes.name == "list_of_changes"
        assert section_data.list_of_changes.type == "list"
        assert isinstance(section_data.list_of_changes.value[0], SectionDataField)
        assert section_data.list_of_changes.value[0].type == "object"
        assert isinstance(
            section_data.list_of_changes.value[0].value["date"], SectionDataField
        )
        assert section_data.list_of_changes.value[0].value["date"].value == "2023-07-24"
        assert section_data.list_of_changes.value[0].value["version"].value == "1.0"
        assert (
            section_data.list_of_changes.value[0].value["description"].value
            == "Description"
        )
        assert section_data.list_of_changes.value[1].value["date"].value == "2023-07-25"
        assert section_data.list_of_changes.value[1].value["version"].value == "2.0"
        assert (
            section_data.list_of_changes.value[1].value["description"].value
            == "New Description"
        )

        # Test setting values
        section_data.title.value = "New Title"
        assert section_data.title.value == "New Title"
        with pytest.raises(ValueError):
            section_data.title.value = 1

        section_data.report_date.value = "2024-08-08"
        assert section_data.report_date.value == "2024-08-08"
        with pytest.raises(ValueError):
            section_data.report_date.value = "new date"

        section_data.draft.value = False
        assert section_data.draft.value == False
        with pytest.raises(ValueError):
            section_data.draft.value = "False"

        section_data.list_of_changes.value[1].value["date"].value = "2025-01-01"
        assert section_data.list_of_changes.value[1].value["date"].value == "2025-01-01"
        with pytest.raises(ValueError):
            section_data.list_of_changes.value[0].value = "Invalid Value"
        with pytest.raises(ValueError):
            section_data.list_of_changes.value[0].value = {"abc": "test"}

        api_test_data = json.loads(self.example_finding_template)

        test_finding_template = FindingTemplate(api_test_data)
        assert test_finding_template.id == "82c5d4c5-b1ff-4d17-9567-c2679df325d5"
        assert (
            test_finding_template.translations[0].data.title == "SQL Injection (SQLi)"
        )
        assert test_finding_template.translations[0].data.references
        assert (
            test_finding_template.translations[0].data.references[0]
            == "https://www.owasp.org/index.php/SQL_Injection_Prevention_Cheat_Sheet"
        )
        assert test_finding_template.translations[1].data.title == "Template Titel"
        assert test_finding_template.translations[1].status == "in-progress"
        assert test_finding_template.translations[1].language == "de-DE"

    @pytest.mark.parametrize(
        "type",
        [
            "list",
            "object",
        ],
    )
    def test_list_unsupported_section_data_field(self, type):
        fld = ProjectDesignField(
            {
                "type": "list",
                "items": {"type": type, "items": {"type": "string"}, "properties": {}},
            }
        )
        with pytest.raises(ValueError):
            SectionDataField(fld, "invalid_value")

    @pytest.mark.parametrize(
        "type,value,invalid_value",
        [
            ("string", "test", 12),
            ("cvss", "test", 12),
            ("markdown", "test", 12),
            ("combobox", "test", 12),
            ("user", "3d503331-77d8-4d5e-b3de-e95875f959c4", 12),
            ("user", "3d503331-77d8-4d5e-b3de-e95875f959c4", "12"),
            ("date", "2023-01-01", "12"),
            ("date", "2023-01-31", "2023-01-32"),
            ("date", "2023-01-31", 12),
            ("number", 12, "12"),
            ("boolean", False, "12"),
            ("boolean", True, "true"),
            ("boolean", False, 1),
            ("boolean", False, 0),
            ("enum", "a", "12"),
            ("enum", "b", 12),
            ("enum", "b", False),
        ],
    )
    def test_list_section_data_field(self, type, value, invalid_value):
        fld = ProjectDesignField(
            {
                "choices": [
                    {"label": "A", "value": "a"},
                    {"label": "B", "value": "b"},
                ],
                "type": type,
            }
        )
        SectionDataField(fld, value)
        with pytest.raises(ValueError):
            SectionDataField(fld, invalid_value)



import argparse
import sys

from reptor.lib.plugins.UploadBase import UploadBase
from reptor.api.NotesAPI import NotesAPI


class File(UploadBase):
    """
    Arguments:
    * --file        files to upload; leave empty for stdin
    * --filename|-fn    filename if file provided via stdin

    """

    meta = {
        "name": "File",
        "summary": "Uploads a file",
    }

    @classmethod
    def add_arguments(cls, parser, plugin_filepath=None):
        super().add_arguments(parser, plugin_filepath)

        parser.add_argument(
            "file",
            nargs="*",
            type=argparse.FileType("r"),
            help="files to upload; leave empty for stdin",
        )
        parser.add_argument(
            "-fn", "--filename", help="filename if file provided via stdin"
        )

    def run(self):
        files = self.reptor.get_config().get("cli").get("file")
        if not files:
            files = [sys.stdin]
        filename = self.reptor.get_config().get("cli").get("filename")
        notename = self.reptor.get_config().get("cli").get("notename")
        parent_notename = None
        icon = None
        if notename:
            parent_notename = "Uploads"
        else:
            notename = "Uploads"
            icon = "📤"
        force_unlock = self.reptor.get_config().get("cli").get("force_unlock")
        no_timestamp = self.reptor.get_config().get("cli").get("no_timestamp")

        self.reptor.api.notes.upload_file(
            files=files,
            filename=filename,
            caption=filename,
            notename=notename,
            parent_notename=parent_notename,
            no_timestamp=no_timestamp,
            force_unlock=force_unlock,
            icon=icon,
        )


loader = File



import os
from pathlib import Path

import pytest

from reptor.lib.plugins.TestCaseToolPlugin import TestCaseToolPlugin

from ..models import Service
from ..Nmap import Nmap


class TestNmap(TestCaseToolPlugin):
    templates_path = os.path.normpath(Path(os.path.dirname(__file__)) / "../templates")

    @pytest.fixture(autouse=True)
    def setUp(self) -> None:
        Nmap.setup_class(
            Path(os.path.dirname(self.templates_path)), skip_user_plugins=True
        )
        self.nmap = Nmap(reptor=self.reptor)

    def _load_grepable_data(self):
        self.nmap.input_format = "grepable"
        filepath = os.path.join(os.path.dirname(__file__), "./data/grepable.txt")
        with open(filepath, "r") as f:
            self.nmap.raw_input = f.read()

    def _load_xml_data(self, xml_file):
        self.nmap.input_format = "xml"
        filepath = os.path.join(os.path.dirname(__file__), f"./data/{xml_file}")
        with open(filepath, "r") as f:
            self.nmap.raw_input = f.read()

    def test_grepable_parse(self):
        result_dict = [
            {
                "ip": "127.0.0.1",
                "hostname": "",
                "port": "80",
                "protocol": "tcp",
                "service": "http",
                "version": "nginx (reverse proxy)",
            },
            {
                "ip": "127.0.0.1",
                "hostname": "",
                "port": "443",
                "protocol": "tcp",
                "service": "ssl/http",
                "version": "nginx (reverse proxy)",
            },
            {
                "ip": "127.0.0.1",
                "hostname": "",
                "port": "8080",
                "protocol": "tcp",
                "service": "",
                "version": "",
            },
        ]
        self._load_grepable_data()
        self.nmap.parse()
        parsed_input_dict = [p.__dict__ for p in self.nmap.parsed_input]
        assert parsed_input_dict == result_dict

    def test_xml_parse_multi_target(self):
        entries = [
            {
                "ip": "142.250.180.228",
                "hostname": "www.google.com",
                "port": "80",
                "protocol": "tcp",
                "service": "http",
                "version": "gws",
            },
            {
                "ip": "142.250.180.228",
                "hostname": "www.google.com",
                "port": "443",
                "protocol": "tcp",
                "service": "https",
                "version": "gws",
            },
            {
                "ip": "34.249.200.254",
                "hostname": "www.syslifters.com",
                "port": "80",
                "protocol": "tcp",
                "service": "http",
                "version": None,
            },
            {
                "ip": "34.249.200.254",
                "hostname": "www.syslifters.com",
                "port": "443",
                "protocol": "tcp",
                "service": "https",
                "version": None,
            },
        ]
        self._load_xml_data("nmap_multi_target.xml")
        self.nmap.parse()
        parsed_input_dict = [p.__dict__ for p in self.nmap.parsed_input]
        for entry in entries:
            assert entry in parsed_input_dict

    def test_multi_notes(self):
        """
        result = {
            "142.250.180.228": {
                "data": [
                    {
                        "hostname": "www.google.com",
                        "ip": "142.250.180.228",
                        "port": "80",
                        "protocol": "tcp",
                        "service": "http",
                        "version": "gws",
                    },
                    {
                        "hostname": "www.google.com",
                        "ip": "142.250.180.228",
                        "port": "443",
                        "protocol": "tcp",
                        "service": "https",
                        "version": "gws",
                    },
                ],
                "show_hostname": True,
            },
            "34.249.200.254": {
                "data": [
                    {
                        "hostname": "www.syslifters.com",
                        "ip": "34.249.200.254",
                        "port": "80",
                        "protocol": "tcp",
                        "service": "http",
                        "version": None,
                    },
                    {
                        "hostname": "www.syslifters.com",
                        "ip": "34.249.200.254",
                        "port": "443",
                        "protocol": "tcp",
                        "service": "https",
                        "version": None,
                    },
                ],
                "show_hostname": True,
            },
        }
        """

        self._load_xml_data("nmap_multi_target.xml")
        self.nmap.multi_notes = True
        self.nmap.parse()
        data = self.nmap.preprocess_for_template()
        assert "34.249.200.254" in data
        assert "142.250.180.228" in data
        assert isinstance(data["34.249.200.254"], dict)
        assert isinstance(data["142.250.180.228"], dict)
        assert "data" in data["34.249.200.254"]
        assert "data" in data["142.250.180.228"]
        assert "show_hostname" in data["34.249.200.254"]
        assert "show_hostname" in data["142.250.180.228"]
        assert len(data["34.249.200.254"]["data"]) == 2
        assert len(data["142.250.180.228"]["data"]) == 2

        assert data["142.250.180.228"]["data"][0].ip == "142.250.180.228"
        assert data["34.249.200.254"]["data"][0].ip == "34.249.200.254"

        self.nmap.format()
        assert isinstance(self.nmap.formatted_input, dict)
        assert "142.250.180.228" in self.nmap.formatted_input
        assert "34.249.200.254" in self.nmap.formatted_input
        assert (
            self.nmap.formatted_input["34.249.200.254"]
            == "| Hostname | IP | Port | Service | Version |\n| ------- | ------- | ------- | ------- | ------- |\n| www.syslifters.com | 34.249.200.254 | 80/tcp | http | n/a |\n| www.syslifters.com | 34.249.200.254 | 443/tcp | https | n/a |"
        )
        assert (
            self.nmap.formatted_input["142.250.180.228"]
            == "| Hostname | IP | Port | Service | Version |\n| ------- | ------- | ------- | ------- | ------- |\n| www.google.com | 142.250.180.228 | 80/tcp | http | gws |\n| www.google.com | 142.250.180.228 | 443/tcp | https | gws |"
        )

    def test_xml_parse_single_target(self):
        entries = [
            {
                "ip": "63.35.51.142",
                "hostname": "www.syslifters.com",
                "port": "80",
                "protocol": "tcp",
                "service": "http",
                "version": None,
            }
        ]
        self._load_xml_data("nmap_single_target_single_port.xml")
        self.nmap.parse()
        parsed_input_dict = [p.__dict__ for p in self.nmap.parsed_input]
        for entry in entries:
            assert entry in parsed_input_dict

    def test_xml_parse_without_hostname(self):
        entries = [
            {
                "ip": "142.251.208.164",
                "hostname": None,
                "port": "80",
                "protocol": "tcp",
                "service": "http",
                "version": None,
            },
            {
                "ip": "142.251.208.164",
                "hostname": None,
                "port": "443",
                "protocol": "tcp",
                "service": "https",
                "version": None,
            },
        ]
        self._load_xml_data("nmap_without_hostname.xml")
        self.nmap.parse()
        parsed_input_dict = [p.__dict__ for p in self.nmap.parsed_input]
        for entry in entries:
            assert entry in parsed_input_dict

    def test_process_parsed_input(self):
        # test without hostname
        self.nmap.parsed_input = list()
        for d in [
            {
                "ip": "127.0.0.1",
                "hostname": "",
                "port": "80",
                "protocol": "tcp",
                "service": "http",
                "version": "nginx (reverse proxy)",
            },
            {
                "ip": "127.0.0.1",
                "hostname": "",
                "port": "443",
                "protocol": "tcp",
                "service": "ssl/http",
                "version": "nginx (reverse proxy)",
            },
        ]:
            s = Service()
            s.parse(d)
            self.nmap.parsed_input.append(s)

        data = self.nmap.preprocess_for_template()
        assert data == {"data": self.nmap.parsed_input, "show_hostname": False}

        # test with hostname
        self.nmap.parsed_input = list()
        for d in [
            {
                "ip": "127.0.0.1",
                "hostname": "",
                "port": "80",
                "protocol": "tcp",
                "service": "http",
                "version": "nginx (reverse proxy)",
            },
            {
                "ip": "127.0.0.1",
                "hostname": "localhost",
                "port": "443",
                "protocol": "tcp",
                "service": "ssl/http",
                "version": "nginx (reverse proxy)",
            },
        ]:
            s = Service()
            s.parse(d)
            self.nmap.parsed_input.append(s)

        data = self.nmap.preprocess_for_template()
        assert data == {"data": self.nmap.parsed_input, "show_hostname": True}

    def test_format_nmap(self):
        result = """| Host | Port | Service | Version |
| ------- | ------- | ------- | ------- |
| 127.0.0.1 | 80/tcp | http | nginx (reverse proxy) |
| 127.0.0.1 | 443/tcp | ssl/http | nginx (reverse proxy) |
| 127.0.0.1 | 8080/tcp | n/a | n/a |"""
        self._load_grepable_data()
        self.nmap.format()
        assert self.nmap.formatted_input == result

        result = """| Hostname | IP | Port | Service | Version |
| ------- | ------- | ------- | ------- | ------- |
| www.syslifters.com | 63.35.51.142 | 80/tcp | http | n/a |"""
        self.nmap.parsed_input = None
        self._load_xml_data("nmap_single_target_single_port.xml")
        self.nmap.format()
        assert self.nmap.formatted_input == result



import pytest

from reptor.lib.errors import MissingArgumentError
from reptor.lib.reptor import Reptor

from ..NotesAPI import NotesAPI


class TestNotesAPI:
    @pytest.fixture(autouse=True)
    def setUp(self):
        self.reptor = Reptor()

    def test_notes_api_init(self):
        # Test valid private note
        self.reptor._config._raw_config["server"] = "https://demo.sysre.pt"
        self.reptor._config._raw_config["cli"] = {"personal_note": True}
        try:
            n = NotesAPI(reptor=self.reptor)
            assert n.personal_note
        except (ValueError, MissingArgumentError):
            self.fail("NotesAPI raised Error")

        # Test valid project note
        self.reptor._config._raw_config["server"] = "https://demo.sysre.pt"
        self.reptor._config._raw_config["cli"] = {"personal_note": False}
        self.reptor._config._raw_config[
            "project_id"
        ] = "2b5de38d-2932-4112-b0f7-42c4889dd64d"
        try:
            n = NotesAPI(reptor=self.reptor)
            assert not n.personal_note
        except (ValueError, MissingArgumentError):
            self.fail("NotesAPI raised Error")

        # Test missing project id and missing personal_note
        self.reptor._config._raw_config["server"] = "https://demo.sysre.pt"
        self.reptor._config._raw_config["cli"] = {"personal_note": False}
        self.reptor._config._raw_config["project_id"] = ""
        with pytest.raises(MissingArgumentError):
            NotesAPI(reptor=self.reptor)

        # Test missing server
        self.reptor._config._raw_config["server"] = ""
        self.reptor._config._raw_config["cli"] = {"personal_note": True}
        with pytest.raises(ValueError):
            NotesAPI(reptor=self.reptor)



import json
import os
from pathlib import Path
from unittest.mock import MagicMock, Mock

import pytest

from reptor.lib.plugins.TestCaseToolPlugin import TestCaseToolPlugin
from reptor.models.Finding import Finding, FindingRaw
from reptor.models.Project import Project
from reptor.models.ProjectDesign import ProjectDesign

from ..ToolBase import ToolBase


class SQLTool(ToolBase):
    def finding_sql(self):
        return {}


class ExampleTool(ToolBase):
    def finding_idor(self):
        return {
            "idor_url": "https://example.com/idor/1337",
        }

    def finding_without_template_empty(self):
        return dict()

    def finding_without_template(self):
        return {"payload": "2=2"}

    def finding_no_finding(self):
        return None


class TestToolbase(TestCaseToolPlugin):
    templates_path = os.path.normpath(
        Path(os.path.dirname(__file__)) / "../tests/data/templates"
    )

    @pytest.fixture(autouse=True)
    def setUp(self) -> None:
        ExampleTool.setup_class(
            Path(os.path.dirname(self.templates_path)), skip_user_plugins=True
        )
        self.example_tool = ExampleTool(reptor=self.reptor)
        self.example_tool.input_format = "json"
        self.example_tool.raw_input = "{}"

        SQLTool.setup_class(
            Path(os.path.dirname(self.templates_path)), skip_user_plugins=True
        )
        self.sql_tool = SQLTool(reptor=self.reptor)
        self.sql_tool.input_format = "json"
        self.sql_tool.raw_input = "{}"

    def test_generate_and_push_findings(self):
        # Patch API query
        self.reptor.api.projects.project = Project(
            {
                "id": "db837c68-ff58-4f63-9161-d2310d71999b",
                "project_type": "c357c387-baff-42ce-8e79-eb0597c3e0e8",
            }
        )
        project_design = """{"id":"c357c387-baff-42ce-8e79-eb0597c3e0e8","created":"2023-08-23T07:28:38.416312Z","updated":"2023-08-23T07:28:38.432044Z","source":"snapshot","scope":"project","name":"Project Design","language":"en-US","details":"","assets":"","copy_of":"7db59c50-275e-4eee-8242-5fef9fbc7abd","lock_info":null,"report_template":"<div>","report_styles":"/* Global styles */","report_fields":{"title":{"type":"string","label":"Title","origin":"core","default":"TODO report title","required":true,"spellcheck":true}},"report_sections":[],"finding_fields":{"title":{"type":"string","label":"Titel","origin":"core","default":"TODO finding title","required":true,"spellcheck":true},"evidence":{"type":"markdown","label":"Evidence","origin":"custom","default":null,"required":true},"payloads":{"type":"list","items":{"type":"string","label":"","origin":"custom","default":null,"required":true,"spellcheck":false},"label":"Payloads","origin":"custom","required":true}},"finding_field_order":[],"finding_ordering":[]}"""
        self.reptor.api.project_designs.project_design = ProjectDesign(
            json.loads(project_design)
        )
        self.reptor.api.templates.search = Mock(return_value=[])

        # Assert "create_finding" is called if no findings exist
        self.reptor.api.projects.get_findings = Mock(return_value=[])
        self.reptor.api.projects.create_finding = MagicMock()

        self.sql_tool.generate_and_push_findings()
        assert self.reptor.api.projects.create_finding.called

        # Assert "create_finding" is not called if finding with same title exists
        finding_raw = FindingRaw({"data": {"title": "SQL issue"}})
        self.reptor.api.projects.get_findings = Mock(return_value=[finding_raw])
        self.reptor.api.projects.create_finding = MagicMock()

        self.sql_tool.generate_and_push_findings()
        assert not self.reptor.api.projects.create_finding.called

        # Assert finding is pushed if no finding from same template exists
        finding = Finding({"template": "12345"})
        self.reptor.api.templates.search = Mock(return_value=[finding])
        self.reptor.api.templates.get_template = Mock(return_value=[finding])
        self.reptor.api.projects.get_findings = Mock(return_value=[finding_raw])
        self.sql_tool.generate_findings = Mock(return_value=None)
        self.sql_tool.findings = [finding]
        self.reptor.api.projects.create_finding_from_template = MagicMock()
        self.reptor.api.projects.update_finding = MagicMock()

        self.sql_tool.generate_and_push_findings()
        assert self.reptor.api.projects.create_finding_from_template.called
        assert self.reptor.api.projects.update_finding.called

        # Assert finding is not pushed if finding from same template exists
        finding_raw = FindingRaw({"template": "12345"})
        finding = Finding(finding_raw)
        self.reptor.api.projects.get_findings = Mock(return_value=[finding_raw])
        self.reptor.api.templates.get_template = Mock(return_value=[finding])
        self.reptor.api.templates.search = Mock(return_value=[finding])
        self.sql_tool.generate_findings = Mock(return_value=None)
        self.sql_tool.findings = [finding]
        self.reptor.api.projects.create_finding_from_template = MagicMock()
        self.reptor.api.projects.update_finding = MagicMock()

        self.sql_tool.generate_and_push_findings()
        assert not self.reptor.api.projects.create_finding_from_template.called
        assert not self.reptor.api.projects.update_finding.called

    def test_no_function_methods(self):
        # There should be no finding_ methods in ToolBase
        finding_method_list = [
            func
            for func in dir(ToolBase)
            if callable(getattr(ToolBase, func)) and func.startswith("finding_")
        ]
        assert len(finding_method_list) == 0
        assert ToolBase._get_finding_methods() == []

    def test_load_local_finding_template(self):
        finding = self.example_tool.get_local_template_data("idor")
        assert finding["data"]["title"] == "IDOR"
        assert (
            finding["data"]["description"]
            == "Insecure Direct Object Reference (IDOR) at <!--{{ idor_url }}-->"
        )
        assert (
            finding["data"]["recommendation"]
            == "<!--{% include 'recommendation.md' %}-->"
        )
        assert len(finding["data"]["references"]) == 2
        assert finding["data"]["references"][0] == "https://reference1.example.com"
        assert finding["data"]["references"][1] == "https://reference2.example.com"
        assert finding["data"]["cvss"] == "My cvss"
        assert finding["data"]["summary"] == "My summary"
        assert finding["data"]["precondition"] == "My precondition"
        assert finding["data"]["impact"] == "My impact"
        assert finding["data"]["short_recommendation"] == "My short recommendation"
        assert finding["data"]["owasp_top10_2021"] == "A01_2021"
        assert finding["data"]["wstg_category"] == "ATHZ"
        assert finding["data"]["retest_notes"] == "My restest notes"
        assert finding["data"]["retest_status"] == "open"
        assert finding["data"]["severity"] == "high"

        # Assert loading with file ext works
        finding_1 = self.example_tool.get_local_template_data("idor.toml")
        assert finding == finding_1

        # Invalid finding
        invalid_finding = self.example_tool.get_local_template_data("invalid_finding")
        assert invalid_finding is None

    def test_generate_findings_with_custom_fields(self):
        # Patch API query
        self.reptor.api.projects.project = Project(
            {
                "id": "db837c68-ff58-4f63-9161-d2310d71999b",
                "project_type": "c357c387-baff-42ce-8e79-eb0597c3e0e8",
            }
        )
        project_design = """{"id":"c357c387-baff-42ce-8e79-eb0597c3e0e8","created":"2023-08-23T07:28:38.416312Z","updated":"2023-08-23T07:28:38.432044Z","source":"snapshot","scope":"project","name":"Project Design","language":"en-US","details":"","assets":"","copy_of":"7db59c50-275e-4eee-8242-5fef9fbc7abd","lock_info":null,"report_template":"<div>","report_styles":"/* Global styles */","report_fields":{"title":{"type":"string","label":"Title","origin":"core","default":"TODO report title","required":true,"spellcheck":true}},"report_sections":[],"finding_fields":{"title":{"type":"string","label":"Titel","origin":"core","default":"TODO finding title","required":true,"spellcheck":true},"evidence":{"type":"markdown","label":"Evidence","origin":"custom","default":null,"required":true},"payloads":{"type":"list","items":{"type":"string","label":"","origin":"custom","default":null,"required":true,"spellcheck":false},"label":"Payloads","origin":"custom","required":true},"references":{"type":"list","items":{"type":"string","label":"","origin":"custom","default":null,"required":true,"spellcheck":false},"label":"References","origin":"custom","required":true}},"finding_field_order":[],"finding_ordering":[]}"""
        self.reptor.api.project_designs.project_design = ProjectDesign(
            json.loads(project_design)
        )
        self.reptor.api.templates.search = Mock(return_value=[])

        # Generate finding with custom fields
        findings = self.sql_tool.generate_findings()
        assert len(findings) == 1
        assert findings[0].data.title.value == "SQL issue"
        assert findings[0].data.evidence.value == "My evidence"
        assert [p.value for p in findings[0].data.payloads.value] == [
            "1=1",
            "2=2",
        ]
        # Assert that empty elements are not added to list
        assert [p.value for p in findings[0].data.references.value] == [
            "https://example.com",
            "https://example.com/additional",
        ]

    def test_generate_findings_with_predefined_fields(self):
        # Patch API query
        self.reptor.api.templates.search = Mock(return_value=[])

        self.example_tool.generate_findings()
        assert len(self.example_tool.findings) == 3

        idor_finding = self.example_tool.findings[0]
        assert isinstance(idor_finding, Finding)
        assert idor_finding.data.title.value == "IDOR"
        assert (
            idor_finding.data.description.value
            == "Insecure Direct Object Reference (IDOR) at https://example.com/idor/1337"
        )
        assert (
            idor_finding.data.recommendation.value
            == "Please fix the IDOR at https://example.com/idor/1337!"
        )
        assert idor_finding.data.cvss.value == "My cvss"
        assert idor_finding.data.summary.value == "My summary"
        assert idor_finding.data.precondition.value == "My precondition"
        assert idor_finding.data.impact.value == "My impact"
        assert idor_finding.data.short_recommendation.value == "My short recommendation"
        assert len(idor_finding.data.references.value) == 2
        assert (
            idor_finding.data.references.value[0].value
            == "https://reference1.example.com"
        )
        assert (
            idor_finding.data.references.value[1].value
            == "https://reference2.example.com"
        )
        assert idor_finding.data.owasp_top10_2021.value == "A01_2021"
        assert idor_finding.data.wstg_category.value == "ATHZ"
        assert idor_finding.data.retest_notes.value == "My restest notes"
        assert idor_finding.data.retest_status.value == "open"
        assert idor_finding.data.severity.value == "high"

        # Test empty finding without template
        no_template_empty_finding = self.example_tool.findings[2]
        assert isinstance(no_template_empty_finding, Finding)
        assert no_template_empty_finding.data.title.value == "Without Template Empty"
        assert no_template_empty_finding.data.description.value == "No description"

        # Test finding without template
        no_template_finding = self.example_tool.findings[1]
        assert isinstance(no_template_finding, Finding)
        assert no_template_finding.data.title.value == "Without Template"
        assert (
            no_template_finding.data.description.value
            == f"```{json.dumps({'payload': '2=2'}, indent=2)}```"
        )



import pytest

from reptor.lib.reptor import Reptor

from ..ProjectsAPI import ProjectsAPI


class TestProjectsAPI:
    @pytest.fixture(autouse=True)
    def setUp(self):
        self.reptor = Reptor()

    def test_project_api_init(self):
        # Test valid init
        self.reptor._config._raw_config["server"] = "https://demo.sysre.pt"
        self.reptor._config._raw_config[
            "project_id"
        ] = "2b5de38d-2932-4112-b0f7-42c4889dd64d"
        try:
            ProjectsAPI(reptor=self.reptor)
        except ValueError:
            self.fail("ProjectsAPI raised ValueError")

        # Test missing server
        self.reptor._config._raw_config["server"] = ""
        with pytest.raises(ValueError):
            ProjectsAPI(reptor=self.reptor)

        # Test missing project id
        self.reptor._config._raw_config["server"] = "https://demo.sysre.pt"
        self.reptor._config._raw_config["project_id"] = ""
        with pytest.raises(ValueError):
            ProjectsAPI(reptor=self.reptor)



from collections import OrderedDict

from reptor.lib.importers.BaseImporter import BaseImporter
from reptor.lib.plugins.ConfBase import ConfBase
from reptor.lib.plugins.ToolBase import ToolBase
from reptor.lib.plugins.UploadBase import UploadBase

SUBCOMMANDS_GROUPS = OrderedDict(
    {
        ConfBase: ("configuration", list()),
        UploadBase: ("uploads", list()),
        ToolBase: ("tools", list()),
        BaseImporter: ("importers", list()),
        "other": ("other", list()),
    }
)



import reptor.subcommands as subcommands
from reptor.lib.plugins.Base import Base
from reptor.lib.importers.BaseImporter import BaseImporter
from reptor.utils.table import make_table


class Importers(Base):
    """

    Use this module to list importers.

    Search for importers based on tags or name

    Create a new importer from a template for the community
    or yourself.
    """

    meta = {
        "name": "Importers",
        "summary": "Show importers to use to import finding templates",
    }

    def __init__(self, **kwargs):
        super().__init__(**kwargs)
        self.arg_search = kwargs.get("search")
        self.arg_new_module = kwargs.get("new")

    @classmethod
    def add_arguments(cls, parser, plugin_filepath=None):
        super().add_arguments(parser, plugin_filepath)
        project_parser = parser.add_argument_group()
        project_parser.add_argument(
            "--search", help="Search for term", action="store", default=None
        )
        # project_parser.add_argument(
        #    "--new", help="Create a new module", action="store_true", default=False
        # )

    def _list(self, importers):
        table = make_table(["Name", "Short Help"])

        for item in importers:
            try:
                name = item.name
            except AttributeError:
                continue
            try:
                summary = item.summary
            except AttributeError:
                summary = ""
            table.add_row(
                name,
                summary,
            )
        self.console.print(table)

    def _search(self, importers):
        """Searches modules"""
        self.log.info(f'Searching for: "{self.arg_search}"')
        results = list()
        for item in importers:
            if self.arg_search in item.tags:
                results.append(item)
                continue

            if self.arg_search in item.name:
                results.append(item)
                continue

        self._list(results)

    # def _create_new_importer(self):
    #    ...

    def run(self):
        if self.arg_search:
            self._search(subcommands.SUBCOMMANDS_GROUPS[BaseImporter][1])
        # elif self.arg_new_module:
        #    self._create_new_importer()
        else:
            self._list(subcommands.SUBCOMMANDS_GROUPS[BaseImporter][1])


loader = Importers
